%this code is provided as-is without any warranty or indemnification of any kind. 
%for any problem please contact me: e-mail adress: valedido@unina.it
%if you are using it please cite:
%Di Donato,V., Jamka. J., Martn-Fernndez J.A. 2020 - Compositional regression-based methods for SST reconstruction
%Alpine and Mediterranean Quaternary,... 

%Bayesian zeroreplacement
%the formula is
%thetaj=nj+s*tj/N+s
%where:
%nj     count of the variable j
%s      the "strength priori" parameter. It controls the weight of a priori distribution 
%       (probability) on the posterior distrubution(probability). If 
%       s=0 the posterior estimate of thetaj is equal to  nj/N and only depends on the observed counts. 
%       with s=0 the a priori tj does not affect the a posteriori estimate.
%tj     t is the parameter of the probability "a priori". With a non "non-informative" model, t 
%       is 1/D (numer of parts).
%N      is the total of the counts 

%use
%[P,C]=zeroconteggi(X,'con',method,varargin)
%THERE IS'N A FIRST COLUMN WITH THE ID OF THE SAMPLES

%[P,C]=zeroconteggi(data,'con','jeffreys')
% X are count data
% t is a number or a vector with the weights (see 'custom') (the default is 1/1D)
% metodo can be      uniforme, s=D (D  il numero di variabili)
%                    jeffreys  s=D/2
%                    noninf    s=1  
%                    custom    to be added as a varargin, in this case t 
%                              can be either a number (non-informative model) or a row vector  
%'con' o 'senza'    refers, respectively, to the presence of absence of a last column with
%                   the total of the rows
%OUTPUT
%C (rescaled counts witout zeros) e P (proportions closed to 1)

%example
%[P,C]=zeroconteggi(dati,'senza','jeffreys')
%

function [P,C]=zeroconteggi(X,con_o_senza_totale,method,varargin)
%sostituzione degli zero con approccio Bayesiano
%la sostituzione  compiuta mediante
%thetaj=nj+s*tj/N+s
%in cui:
%nj     conteggi della variabile j
%s       il parametro di "strength priori". E' una misura che controlla l'effetto (o "peso") che ha 
%       la distribuzione a priori (probabilit) nella distribuzione a posteriori (probabilit). Notare che se 
%       s=0 allora la stima a posteriori di thetaj  uguale a nj/N e dipende solo dai conteggi osservati. 
%       per s=0 la t a priori non ha effetto sulla stima a posteriori.
%tj     t  il parametro della probabilit attesa "a priori". Se si assume un modello "non-informativo", t 
%        pari a 1/D (numero di categorie) 
%N       il totale dei conteggi della righe (N.B. dei conteggi originali, non delle sole variabili selezionate) 
%
%Per ogni osservazione (righe del data set di parti per uno) si fa la seguente sostituzione degli zeri  
% 0  sostituito da s*tj/(tot+s),
% poi si riaggiustano i valori non  nulli (parti per 1) della riga secondo
% sostituzione moltiplicativa:
% x  sostituito da x*(1-sum_of_imputed_values_in_row)
%Dopo la sostituzione si ottiene il set di dati P (parti per uno) senza zeri.
%Si pu anche ottenere un data set C (counts) senza zeri moltiplicando ogni riga di P per il totale dei conteggi 
%delle righe di X. Questa strategia ha il vantaggio di preservare i conteggi iniziali.  
%Con approccio Bayesiano, ci significa applicare la formula
%thetaj=(nj+s*tj/N+s)*N
%che chiaramente  una stima di N*thetaj.
%In pratica un valore zero (nj=0) nel data set di conteggi  sostituito da [(1/D)/(tot+1)]*tot.
%Naturally, for the cases where tot is a larger value (tend to infinity) the replaced value tends to 1/D.  
%In our cases (modern and fossil) the total counts are relative larger, then the zero values 
%are all replaced by approximately s*tj. In our case we are taking s=1 and tj=1/D. 
%That is a priori all the components have the same probability (1/D). That is the reason why 
%all the zeros values for each components are replaced by 
%the approximately same value (in v2 by 1/20=0.05, in v5 by 1/16=0.06). 
%
%NON C'E' COLONNA DI INTESTAZIONE
%[P,C]=zeroconteggi(X,'con',method,varargin)
%
%[P,C]=zeroconteggi(data,'con','jeffreys')
% X sono i dati espressi da conteggi
% t  un numero o un vettore per i pesi (vedi 'custom')
% metodo pu essere: uniforme, s=D (D  il numero di variabili)
%                    jeffreys  s=D/2
%                    noninf    s=1  
%                    custom    da inserire come varargin, in questo caso t 
%                              pu essere  un numero (non-informative model) o un vettore riga  
%restituisce C (conteggi sostituiti) e P (chiusa a 1)
%
%'con' o 'senza' fa riferimento alla presenza o meno di una colonna finale
%con il totale delle righe
%esempio di utilizzo
%[P,C]=zeroconteggi(dati,'senza','jeffreys')
switch con_o_senza_totale
    %caso in cui la matrice X contenga solo conteggi, senza una colonna
    %finale di totali
case 'senza'
[N D]=size(X);
%alloca le matrici di uscita
C=zeros(N,D); %matrice di conteggi
P=zeros(N,D); %matrice di proporzioni
%crea un vettore con la somma di ciascuna riga, che viene anche
%utilizzato per il denominatore della sostitutzione 
T=sum(X,2);
%calcola le proporzioni per ogni variabile
    for i=1:N
    prop(i,:)=X(i,:)./T(i,1);
    end
    %caso in cui la matrice X contenga come ultima colonna i totali 
case 'con'
[N D]=size(X);
%alloca le matrici di uscita
C=zeros(N,D-1); %matrice di conteggi
P=zeros(N,D-1); %matrice di proporzioni    
%utilizza come T l'ultima riga di X
T=X(:,end);
%calcola le proporzioni per ogni variabile
    for i=1:N
    prop(i,:)=X(i,:)./T(i,1);
    end
end
%determina il valore da usare per t, per default si ha 1/D se nargin=4 si
%usa varagin{1}
    if nargin==3 
    t=1/D;
    elseif nargin==4
    t=varargin{1};
    end
%sceglie il metodo
switch method
case 'noninf'
s=1
case 'uniforme'
s=D
case 'jeffreys'
s=D/2;
%%%%    case 'custom'
%%%%    s=varargin{1};
end
%verifica e conta quanti zeri ci sono in ogni riga
switch con_o_senza_totale
case 'senza'
Z=zeros(N,D);
    for i=1:N
        for j=1:D
            if X(i,j)==0
               Z(i,j)=1;
            else
            end
        end
    end
Somma=zeros(N,1);
    for i=1:N
    Somma(i)=sum(Z(i,:));
    end
case 'con'
Z=zeros(N,D-1);
    for i=1:N
        for j=1:D-1
            if X(i,j)==0
            Z(i,j)=1;
            else
            end
        end
    end
Somma=zeros(N,1);
    for i=1:N
    Somma(i)=sum(Z(i,:));
    end
end
%calcola i valori, sostituendo gli zeri e riadattando gli altri secondo
%multiplicative replacement. 
switch con_o_senza_totale
case 'senza'
    for i=1:N
        for j=1:D
            if X(i,j)==0
            P(i,j)=(s*t./(T(i)+s));
        %C(i,j)=t*s./(T(i,1)+s);
        %C(i,j)=P(i,j).*T(i,1);
            else
            P(i,j)=prop(i,j)*(1-Somma(i)*(s*t./(T(i)+s)));
        %C(i,j)=X(i,j);
        %C(i,j)=P(i,j).*T(i,1);
            end
        end
    end
case 'con'
    for i=1:N
        for j=1:D-1
            if X(i,j)==0
            P(i,j)=(s*t./(T(i)+s));
        %C(i,j)=t*s./(T(i,1)+s);
        %C(i,j)=P(i,j).*T(i,1);
            else
            P(i,j)=prop(i,j)*(1-Somma(i)*(s*t./(T(i)+s)));
        %C(i,j)=X(i,j);
        %C(i,j)=P(i,j).*T(i,1);
            end
        end
    end
end
    %crea la matrice dei conteggi C
    for i=1:N
    C(i,:)=P(i,:).*T(i);
    end
end
 %considera se t  un numero o un vettore
         %   [u v]=size(t)    
          %  if v=1       
          % elseif v=D 
%nota sulla versione ora  conforme a Martin
%la versione 1.4 non funzionava ora  corretta