%this code is provided as-is without any warranty or indemnification of any kind. 
%for any problem please contact me: e-mail adress: valedido@unina.it
%if you are using it please cite:
%Di Donato,V., Jamka. J., Martn-Fernndez J.A. 2020 - Compositional regression-based methods for SST reconstruction
%Alpine and Mediterranean Quaternary,... 


%Zero replacement
%X is the data matrix (either counts or proportions). 
%R is a replacement values (for 'simple' and 'multi' methods) with different for each 
%observation ('simple') or variable ('multi') 
%method:    'simple' replace the R value, the close the total
%           'multicol' replaces the zeros with a value equal to R times the
%           minimum of each column (i.e. in geochemical data the non detection limit, which can be different for the different elements )
%           'multirow' replaces the zeros with a value equal to R times the
%                      minimum of each row
%esempio
%      B=zerorep(X,0.65,'multirow')


%X sono i dati (possono essere conteggi o dati gi chiusi). 
%R  un valore di rimpiazzo o un vettore (per i metodi 'simple' e 'multi') con valori differenti per ciascuna
%osservazione ('simple') o variabile ('multi') 
%method:    'simple' rimpiazza un valore nei conteggi, poi richiude le osservazioni
%           'minimo' rimpiazza gli zeri con un valore pari a R * (il minimo
%              della colonna)
%           'multicol' multiplicative zero replacement (minimo delle colonne
%           'multirow' minimo delle righe
%esempio
%      B=zerorep(X,0.65,'multirow')
function [B]=zerorep(X,R,method)
[n p]=size(X);
s=sum(X,2);
switch method
    case 'simple'
        for i=1:n
            for j=1:p
                if X(i,j)==0
                    if size(R)==[n 1] 
                    X(i,j)=R(n);
                    elseif size(R)==[1 1]       
                    X(i,j)=R;
                    end
                else                
                %X(i,:)=X(i,:)
                end
            end
        end
        s2=sum(X,2);
        for i=1:n
            B(i,:)=X(i,:)./s2(i);
        end
    case 'minimo'    
    %se non lo sono gi, chiude i dati a 1
    B=zeros(n,p);
    for i=1:n
    B(i,:)=X(i,:)./s(i);
    end   
    %sostituisce gli zeri con 9999
        for i=1:n
            for j=1:p
                if B(i,j)==0
                B(i,j)=9999;
                else
                B(i,j)=B(i,j);
                end
            end    
        end
        %calcola il minimo di ogni variabile
        M=min(B);
        %sostituisce i valori 9999 con il minimo
        for i=1:n
            for j=1:p
                if B(i,j)==9999
                B(i,j)=R*M(1,j);    
                else
                B(i,j)=B(i,j);
                end
            end
        end
    case 'multicol'
    %se non lo sono gi, chiude i dati a 1
    B=zeros(n,p);
    for i=1:n
    B(i,:)=X(i,:)./s(i);
    end
       %crea una matrice di 0 e 1    
    %z=zeros(n,p);
    %    for i=1:n
    %        for j=1:p
    %            if B(i,j)~=0;
    %            z(i,j)=0;
    %            else
    %            z(i,j)=1 ;   
    %            end
    %        end
    %    end
     %vettore con elementi pari ai valori nulli di ogni osservazione
     %q=sum(z,2);
     %sostituisce gli zeri con 9999
     for i=1:n
        for j=1:p
            if B(i,j)==0
            B(i,j)=9999;
            else
            B(i,j)=B(i,j);
            end
        end    
     end
     %calcola il minimo di ogni variabile (colonne)
     M=min(B);
     %sostituisce i valori 9999 con il minimo
     for i=1:n
         for j=1:p
             if B(i,j)==9999
                if size(R)==[p 1] 
                B(i,j)=R(j);
                elseif size(R)==[1 1]       
                B(i,j)=R*M(j);
                end
                else             
             end
         end
     end
     %calcola la somma dei valori sostituiti per ogni osservazione
     g=sum(B,2)-1;   
    %multiplicative replacement
     for i=1:n
         for j=1:p
             if B(i,j)==R*M(j)    
             B(i,j)=B(i,j);
             else        
             B(i,j)=B(i,j)*(1-g(i));   
             end
         end
     end
    case 'multirow'
    B=zeros(n,p);    
    %se non lo sono gi, chiude i dati a 1
        for i=1:n
        B(i,:)=X(i,:)./s(i);
        end
       %crea una matrice di 0 e 1    
    %z=zeros(n,p);
    %    for i=1:n
    %        for j=1:p
    %            if B(i,j)~=0;
    %            z(i,j)=0;
    %            else
    %            z(i,j)=1 ;   
    %            end
    %        end
    %    end
     %vettore con elementi pari ai valori nulli di ogni osservazione
     %q=sum(z,2);
     %sostituisce gli zeri con 9999
     for i=1:n
         for j=1:p
                if B(i,j)==0
                B(i,j)=9999;
                else
                B(i,j)=B(i,j);
                end
         end    
     end
        %calcola il minimo di ogni riga
        M=min(B')
        M=M'
        %sostituisce i valori 9999 con il minimo di ogni riga * R
        %se R  un matrice
    if size(R)==[n 1]
       RM=R.*M
        for i=1:n
            for j=1:p
                if B(i,j)==9999
                B(i,j)=RM(i,1);
                else
                end
            end
        end
        %se invece R  solo un numero
    elseif size(R)==[1 1]    
        for i=1:n
            for j=1:p
                if B(i,j)==9999
                B(i,j)=R.*M(i,1);
                else             
                end
            end
        end
    end
    %calcola la somma dei valori sostituiti per ogni osservazione (riga)
    g=sum(B,2)-1;   
    %multiplicative replacement
    if size(R)==[n 1]
        for i=1:n
            for j=1:p
                if B(i,j)==RM(i,1)    
                B(i,j)=B(i,j);
                else        
                B(i,j)=(1-g(i)*B(i,j));
                end
            end         
        end
    elseif size(R)==[1 1]    
         for i=1:n
            for j=1:p
                if B(i,j)==R*M(i)    
                B(i,j)=B(i,j);
                else        
                B(i,j)=B(i,j)*(1-g(i));   
                end
            end
        end         
    end
end
end
        %     if B(i,j)~=0
        %            if size(R)==[p 1] 
        %            B(i,j)=B(i,j).*(1.-R(j)*q(i));
        %     elseif size(R)==[1 1]  
        %            B(i,j)=B(i,j).*(1-R*M(j)*q(i));
        %            end
        %        end
        %     end 
        %end
        %end
  