%this code is provided as-is without any warranty or indemnification of any kind. 
%for any problem please contact me: e-mail adress: valedido@unina.it
%if you are using it please cite:
%Di Donato,V., Jamka. J., Martn-Fernndez J.A. 2020 - Compositional regression-based methods for SST reconstruction
%Alpine and Mediterranean Quaternary,... 

%help:
%metodo             can be: 'pls' (CodaPLS)
%                           'pcregr' (CodaPCR)
%                           'robustpcr' (Robust Coda PCR)
%                           'pcquadr' (CodaPCR with linear and squared terms for each component (predictor))
%                           'robustpls' (Robust PLS 
%                           Hubert, M., and Vanden Branden, K. (2003),
%                           "Robust Methods for Partial Least Squares Regression",
%                           Journal of Chemometrics, 17, 537-549.
%                           N.B. this method needs the rsimpls function!, 
%                           check here 
%                           https://github.com/duncombe/matlab/blob/master/LIBRA/rsimpls.m)
%                           in the help there are references to be
%                           considered
%                                                    
%envir                      is a matrix with the environmental parameters
%                           included in the modern database
%mod                        is a matrix with the environmental parameters
%                           included in the modern database
%fossil                     is a matrix with the fossil assemblages. The
%                           first column includes ID or cm or age
%N.B.       envir, modm, fossil can be either matrices or cell arrays (i.e.
%           envir{1,1}, envir{2,1},...,envir{gr,1}) if the modern data are made of separate groups
%           If the groups involve different subcompositions, a cell array
%           with, respectively, the full modern ({1,1})and fossil ({2,1}) full compositions
%           must be added as a varargin{1}.
%           It is needed to perform the discriminant analysis.
%           
%tipodati                   can be 'datiilr' (ilr coordinates) or 'daticlr'
%                           (clr coordinates)
%Mtext                      is a text string with the column headers on the
%                           modern environmental variables (ID + environmental variables)
%grafici                    can be 'singoli' (single graphs of
%                           reconstructed values) or 'subplots' (this option needs the plotricreg function)
%Zscores                    can be 'yes' (standardizes the environmental parameters) or
%                           'no' (leave the environmental parameters as they are)
%
%N.B. In order to make this code run, you need the plsregress, pca, fitlm, classify MATLAB
%functions
%
%output: 
%ricfit                     a matrix with reconstructed values for a core
%                           dataset. The first column includes ages or depth (as in the fossil input
%                           matrix)
%validation                 validation.RMSE is the root mean square error
%                           obtained with cross-validation
%                           validation.R2 the R square. For grouped data it
%                           a pooled R2 ()
%                           validation.R2overall is an overall R2 for
%                           grouped data (an empty matrix for non grouped
%                           data)
%                           validation.environ the modern environmental data
%                           validation.environmentfitted the fitted modern environmental data           
%                           

%associazionifossili comprende una prima colonna con age depth, serve per i
%grafici
%se ci sono gruppi nelle associazionimoderne e nei dati moderni, vanno inseriti come cell arrays
%(datiambientali{1,1}, datiambientali{2,1}etc and
%associazionimoderne{1,1},associazionimoderne{2,1} etc etc
%varargin{1} sono le associazioni moderne con la composizione piena che DEVONO essere aggiunte (senza colonna di intestazione)
%quando mod  diviso in gruppi (che possono anche essere subcomposizioni)

%N.B. se i gruppi nel moderno implicano subcomposizioni diverse, bisogna inserire anche
%il fossile in termini di subcomposizione, tipo {1,1} e {2,1}, oltrech,
%obbligatoriamente, il moderno e il fossile completo (varargin{1})su cui effettuare l'analisi
%discriminante. In particolare, varargin{1}  inserito come cell array: varargin{1}{1,1} il moderno
%varargin{1}{2,1}  il fossile: 

function [ricfit,validation]=codatransfer(metodo,envir,mod,fossil,tipodati,Mtext,grafici,Zscores,varargin)
switch Zscores
    case 'yes'
        medie=mean(envir);
        deviazstd=std(envir);
        envir=zscore(envir);
    case 'no'
    otherwise
    error('Zscores can be yes or no')
end
%[nf,pf]=size(fossil);
%comincia le analisi con pls gruppo singolo
switch metodo
case 'pls'
    if iscell(envir)==0
        [~,pe]=size(envir);
        y=envir;                   % Response (SST annual)
        X=mod;              % predittori
        [~,pm]=size(X);
        [XL,yl,XS,YS,beta,PCTVAR,MSE] = plsregress(X,y,pm);
        %varianza spiegata
        figure
        plot(1:pm,cumsum(100*PCTVAR(2,:)),'-bo');
        xlabel('Number of PLS components');
        ylabel('Percent Variance Explained in y');
        figure
        [axes,h1,h2] = plotyy(0:pm,MSE(1,:),0:pm,MSE(2,:));
        set(h1,'Marker','o')
        set(h2,'Marker','o')
        legend('MSE Predictors','MSE Response')
        xlabel('Number of Components')
        prompt = 'How many components?';
        ncomp = input(prompt)   
        %ripete con ncomp componenti
        [XL,yl,XS,YS,beta,PCTVAR,MSE,stats] = plsregress(X,y,ncomp);
        yfit=[ones(size(X,1),1) X]*beta;
        %se c' un solo parametro ripete una sola volta
            %R2
            for j=1:pe
            %misurati vs stimati
            TSS{1,j}=sum((y(:,j)-mean(y(:,j))).^2);
            RSS{1,j}=sum((y(:,j)-yfit(:,j)).^2);
            DSS{1,j}=sum((yfit(:,j)-mean(y(:,j))).^2);
            DSS{1,j}./TSS{1,j}
            R2{1,j}=1-RSS{1,j}/TSS{1,j}
        %RMSE
            RMSE(1,j)=(sum((y(:,j)-yfit(:,j)).^2)/(size(y(:,j),1)))^0.5                
            end
%           RMSE=MSE(2,:).^0.5
        R2overall=[];
    %se ci sono gruppi nel moderno
    elseif iscell(envir)==1
            [gr,~]=size(envir)
            [~,pe]=size(envir{1});
            %ripete per tutti i gr gruppi
            for i=1:gr
                %GRUPPO
                ygr{i,1}=envir{i,1}
                Xgr{i,1}=mod{i,1};              % predittori
                [~,pm]=size(Xgr{i});
                [XL,yl,XS,YS,beta{i,1},PCTVAR,MSE]=plsregress(Xgr{i,1},ygr{i,1},pm);
                %varianza spiegata
                %varianza spiegata
                figure
                plot(1:pm,cumsum(100*PCTVAR(2,:)),'-bo');
                xlabel('Number of PLS components');
                ylabel('Percent Variance Explained in y');
                figure
                [axes,h1,h2] = plotyy(0:pm,MSE(1,:),0:pm,MSE(2,:));
                set(h1,'Marker','o')
                set(h2,'Marker','o')
                legend('MSE Predictors','MSE Response')
                xlabel('Number of Components')
                prompt='How many components?';
                ncomp=input(prompt)
                %ripete con ncomp componenti
                [XL,yl,XS,YS,beta{i,1},PCTVAR,MSE,stats]=plsregress(Xgr{i,1},ygr{i,1},ncomp);
                yfitgr{i,1}=[ones(size(Xgr{i,1},1),1) Xgr{i,1}]*beta{i,1};
                %se c' un solo parametro ambientale pe  1  non ripete
                %R2
                for j=1:pe
                %misurati vs stimati
%                figure
%                plot(ygr{i,j},yfitgr{i,j},'.')
                TSS{i,j} = sum((ygr{i,1}(:,j)-mean(ygr{i,1}(:,j))).^2);
                RSS{i,j} = sum((ygr{i,1}(:,j)-yfitgr{i,1}(:,j)).^2);
                DSS{i,j} = sum((yfitgr{i,1}(:,j)-mean(ygr{i,1}(:,j))).^2);
                DSS{i,j}./TSS{i,j}
                R2gr{i,j}=1-RSS{i,j}/TSS{i,j}
                end
            end
            %Rsquared_complessivo dei gruppi (y sono i dati ambientali)
            %se sono pi variabili, y{1}  una matrice
            %Se c' una sola variabile non ripete
%           figure
            for j=1:pe
            %DSS e TSS dei gruppi   
            DSSgr=DSS{1,j};
            TSSgr=TSS{1,j};
                for i=2:gr
                DSSgr=DSSgr+DSS{i,j};
                TSSgr=TSSgr+TSS{i,j};
                end   
            R2(1,j)=DSSgr./TSSgr
            %y e yfit
            y=ygr{1,1};
            yfit=yfitgr{1,1};           
            for i=2:gr
            y=[y; ygr{i,1}];
            yfit=[yfit; yfitgr{i,1}];
            end
            %R2 complessiva (quadrato di r)
            TSStutti=sum((y(:,1)-mean(y(:,1))).^2);
            RSStutti=sum((y(:,1)-yfit(:,1)).^2);
            DSStutti=sum((yfit(:,1)-mean(y(:,1))).^2);
            R2overall(1,j)=1-RSStutti/TSStutti
            %RMSE:
            RMSE(1,j)=(sum((y(:,1)-yfit(:,1)).^2)/(size(y,1))).^0.5
            end
    end
case 'robustpls'
    if iscell(envir)==0
        [~,pe]=size(envir);
        y=envir;            % Response (SST annual)
        X=mod;              % predittori
        [~,pm]=size(X);
        %resultgr1=rsimpls(X,y,'k',pm,'plots',1);
        result=rsimpls(X,y,'rmsecv',1,'plots',0);
        %misurati vs stimati
        %Rsquared_dei due gruppi
        yfit=result.fitted;
        for j=1:pe
        %se c' un solo parametro ripete una sola volta
        TSS{1,j} = sum((y(:,1)-mean(y(:,1))).^2);
        RSS{1,j} = sum((y(:,1)-yfit(:,1)).^2);
        DSS{1,j} = sum((yfit(:,1)-mean(y(:,1))).^2);
        DSS{1,j}./TSS{1,j}
        R2{1,j}=1-RSS{1,j}/TSS{1,j}
        %RMSE
        RMSE=(sum((y(:,j)-yfit(:,j)).^2)/(size(y(:,j),1)))^0.5                
        end
%       RMSE=MSE(2,:).^0.5
        R2overall=[];
    %se ci sono gruppi nel moderno
    elseif iscell(envir)==1
            [gr,~]=size(envir)
            [~,pe]=size(envir{1});
            %ripete per tutti i gr gruppi
            for i=1:gr
                %GRUPPO
                ygr{i,1}=envir{i,1}
                Xgr{i,1}=mod{i,1};              % predittori
                [~,pm]=size(Xgr{i});
                result=rsimpls(Xgr{i,1},ygr{i,1},'rmsecv',1,'plots',0);
                yfitgr{i,1}=result.fitted;
                %misurati vs stimati
                %se c' un solo parametro ambientale pe  1  non ripete
                %R2
                for j=1:pe 
                TSS{i,j} = sum((ygr{i,1}(:,j)-mean(ygr{i,1}(:,j))).^2);
                RSS{i,j} = sum((ygr{i,1}(:,j)-yfitgr{i,1}(:,j)).^2);
                DSS{i,j} = sum((yfitgr{i,1}(:,j)-mean(ygr{i,1}(:,j))).^2);
                DSS{i,j}./TSS{i,j}
                R2gr{i,j}=1-RSS{i,j}/TSS{i,j}
                end
            end
            %Rsquared_complessivo dei gruppi (y sono i dati ambientali)
            %se sono pi variabili, y{1}  una matrice
            %Se c' una sola variabile non ripete
            y=ygr{1,1};
            yfit=yfitgr{1,1};
            for i=2:gr
            y=[y; ygr{i,1}];
            yfit=[yfit; yfitgr{i,1}];
            end
            for j=1:pe
            %DSS e TSS dei gruppi   
            DSSgr=DSS{1,j};
            TSSgr=TSS{1,j};
                for i=2:gr
                DSSgr=DSSgr+DSS{i,j};
                TSSgr=TSSgr+TSS{i,j};
                end   
            R2(1,j)=DSSgr./TSSgr
            %R2 complessiva (quadrato di r)
            TSStutti=sum((y(:,1)-mean(y(:,1))).^2);
            RSStutti=sum((y(:,1)-yfit(:,1)).^2);
            DSStutti=sum((yfit(:,1)-mean(y(:,1))).^2);
            R2overall(1,j)=1-RSStutti/TSStutti
                %RMSEpooled:
            RMSE(1,j)=(sum((y(:,1)-yfit(:,1)).^2)/(size(y,1))).^0.5
            end
    end
case 'pcreg' 
     if iscell(envir)==0
        [~,pe]=size(envir);
        y=envir;                   % Response (SST annual)
        X=mod;              % predittori
        [~,pm]=size(X);
        [PCALoadings,PCAScores,PCAVar] = pca(X,'Economy',false);
        figure
        plot(1:pm,(100*cumsum(PCAVar)/sum(PCAVar)),'-bo');
        hold on
        plot(1:pm,(100*(PCAVar)/sum(PCAVar)),'-r*');
        xlabel('Number of components');
        ylabel('Percent Variance Explained');
        hold off
        %dal grafico valutare i componenti
        prompt = 'How many components?';
        ncomp = input(prompt)   
        %procede con ncomp componenti, se pe  1 non ripete
            for j=1:pe           
            mdlPCR=fitlm(PCAScores(:,1:ncomp),y(:,j),'linear');
            %nota: in table2array(mdl.Coefficients(2:end,1)) si trovano i coefficenti
            %come generati da regress (il primo elemento  l'intercetta)
            %i comandi seguenti servono a rappresentare il modello in termini di
            %variabili originali e non centrate.
            %come alternativa, si possono fattorizzare le osservazioni delle carote:
            %(daticarota-mediacolonnedatabase)*PCAloadings
            betap{1,j} = PCALoadings(:,1:ncomp)*table2array(mdlPCR.Coefficients(2:end,1));
            betap{1,j} = [mean(y(:,j)) - mean(X)*betap{1,j}; betap{1,j}];
            yfit(:,j)=[ones(size(X,1),1) X]*betap{1,j};
            beta(:,j)=betap{1,j};
            R2{1,j}=mdlPCR.Rsquared
            %RMSO
            RMSE(1,j)=(sum((y(:,j)-yfit(:,j)).^2)/(size(y(:,j),1)))^0.5
            end
            R2overall=[];
    %se invece ci sono gruppi
    elseif iscell(envir)==1      
        [gr,~]=size(envir)
        [~,pe]=size(envir{1});
        %ripete per tutti i gr gruppi
        for i=1:gr
            %GRUPPO
            ygr{i,1}=envir{i,1}
            Xgr{i,1}=mod{i,1};              % predittori
            [~,pm]=size(Xgr{i,1});
            [PCALoadings,PCAScores,PCAVar] = pca(Xgr{i,1},'Economy',false);
            figure
            plot(1:pm,(100*cumsum(PCAVar)/sum(PCAVar)),'-bo');
            hold on
            plot(1:pm,(100*(PCAVar)/sum(PCAVar)),'-r*');
            xlabel('Number of components');
            ylabel('Percent Variance Explained');
            hold off
            %dal grafico valutare i componenti
            prompt = 'How many components?';
            ncomp=input(prompt)   
            %procede con ncomp componenti per tutti i pe (potrebbe
            %essere anche 1)
            for j=1:pe           
                mdlPCR=fitlm(PCAScores(:,1:ncomp),ygr{i,1}(:,j),'linear');
                betap{i,j} = PCALoadings(:,1:ncomp)*table2array(mdlPCR.Coefficients(2:end,1));
                betap{i,j} = [mean(ygr{i,1}(:,j)) - mean(Xgr{i,1})*betap{i,j}; betap{i,j}];
                yfitgr{i,1}(:,j)=[ones(size(Xgr{i,1},1),1) Xgr{i,1}]*betap{i,j};
                beta{i,1}(:,j)=betap{i,j}
                TSS{i,j} = sum((ygr{i,1}(:,j)-mean(ygr{i,1}(:,j))).^2);
                RSS{i,j} = sum((ygr{i,1}(:,j)-yfitgr{i,1}(:,j)).^2);
                DSS{i,j} = sum((yfitgr{i,1}(:,j)-mean(ygr{i,1}(:,j))).^2);
                DSS{i,j}./TSS{i,j}
                R2gr{i,j}=1-RSS{i,j}/TSS{i,j}
            end
        end
            %Rsquared_complessivo dei gruppi (y sono i dati ambientali)
            %se sono pi variabili, y{1}  una matrice
            %Se c' una sola variabile non ripete
            y=ygr{1,1};
            yfit=yfitgr{1,1};
                for i=2:gr
                y=[y; ygr{i,1}];
                yfit=[yfit; yfitgr{i,1}];
                end
            for j=1:pe
            %DSS e TSS dei gruppi   
            DSSgr=DSS{1,j};
            TSSgr=TSS{1,j};
                for i=2:gr
                DSSgr=DSSgr+DSS{i,j};
                TSSgr=TSSgr+TSS{i,j};
                end   
            R2(1,j)=DSSgr./TSSgr
            %R2 complessiva (quadrato di r)
            TSStutti=sum((y(:,1)-mean(y(:,1))).^2);
            RSStutti=sum((y(:,1)-yfit(:,1)).^2);
            DSStutti=sum((yfit(:,1)-mean(y(:,1))).^2);
            R2overall(1,j)=1-RSStutti/TSStutti
            %RMSE:
            RMSE(1,j)=(sum((y(:,1)-yfit(:,1)).^2)/(size(y,1))).^0.5
            end
     end
case 'robustpcr'
     if iscell(envir)==0
        [~,pe]=size(envir);
        y=envir;                   % Response (SST annual)
        X=mod;              % predittori
        [~,pm]=size(X);
        [PCALoadings,PCAScores,PCAVar] = pca(X,'Economy',false);
        figure
        plot(1:pm,(100*cumsum(PCAVar)/sum(PCAVar)),'-bo');
        hold on
        plot(1:pm,(100*(PCAVar)/sum(PCAVar)),'-r*');
        xlabel('Number of components');
        ylabel('Percent Variance Explained');
        hold off
        %dal grafico valutare i componenti
        prompt = 'How many components?';
        ncomp = input(prompt)   
        %procede con ncomp componenti, se pe  1 non ripete
            for j=1:pe           
            mdlPCR=fitlm(PCAScores(:,1:ncomp),y(:,j),'linear','RobustOpts','on');
            %nota: in table2array(mdl.Coefficients(2:end,1)) si trovano i coefficenti
            %come generati da regress (il primo elemento  l'intercetta)
            %i comandi seguenti servono a rappresentare il modello in termini di
            %variabili originali e non centrate.
            %come alternativa, si possono fattorizzare le osservazioni delle carote:
            %(daticarota-mediacolonnedatabase)*PCAloadings
            betap{1,j} = PCALoadings(:,1:ncomp)*table2array(mdlPCR.Coefficients(2:end,1));
            betap{1,j} = [mean(y(:,j)) - mean(X)*betap{1,j}; betap{1,j}];
            yfit(:,j)=[ones(size(X,1),1) X]*betap{1,j};
            beta(:,j)=betap{1,j};
            R2{1,j}=mdlPCR.Rsquared
            %RMSO
            RMSE(1,j)=(sum((y(:,j)-yfit(:,j)).^2)/(size(y(:,j),1)))^0.5
            %misurati vs stimati
            end
            R2overall=[];
            %se invece ci sono gruppi
    elseif iscell(envir)==1      
        [gr,~]=size(envir)
        [~,pe]=size(envir{1});
        %ripete per tutti i gr gruppi
        for i=1:gr
            %GRUPPO
            ygr{i,1}=envir{i,1}
            Xgr{i,1}=mod{i,1};              % predittori
            [~,pm]=size(Xgr{i,1});
            [PCALoadings,PCAScores,PCAVar] = pca(Xgr{i,1},'Economy',false);
            figure
            plot(1:pm,(100*cumsum(PCAVar)/sum(PCAVar)),'-bo');
            hold on
            plot(1:pm,(100*(PCAVar)/sum(PCAVar)),'-r*');
            xlabel('Number of components');
            ylabel('Percent Variance Explained');
            hold off
            %dal grafico valutare i componenti
            prompt = 'How many components?';
            ncomp=input(prompt)   
            %procede con ncomp componenti per tutti i pe (potrebbe
            %essere anche 1)
            for j=1:pe           
                mdlPCR=fitlm(PCAScores(:,1:ncomp),ygr{i,1}(:,j),'linear');
                betap{i,j} = PCALoadings(:,1:ncomp)*table2array(mdlPCR.Coefficients(2:end,1));
                betap{i,j} = [mean(ygr{i,1}(:,j)) - mean(Xgr{i,1})*betap{i,j}; betap{i,j}];
                yfitgr{i,1}(:,j)=[ones(size(Xgr{i,1},1),1) Xgr{i,1}]*betap{i,j};
                beta{i,1}(:,j)=betap{i,j}
                TSS{i,j} = sum((ygr{i,1}(:,j)-mean(ygr{i,1}(:,j))).^2);
                RSS{i,j} = sum((ygr{i,1}(:,j)-yfitgr{i,1}(:,j)).^2);
                DSS{i,j} = sum((yfitgr{i,1}(:,j)-mean(ygr{i,1}(:,j))).^2);
                DSS{i,j}./TSS{i,j}
                R2gr{i,j}=1-RSS{i,j}/TSS{i,j}
            end
        end
            %Rsquared_complessivo dei gruppi (y sono i dati ambientali)
            %se sono pi variabili, y{1}  una matrice
            %Se c' una sola variabile non ripete
%            figure
            y=ygr{1,1};
            yfit=yfitgr{1,1};
            for i=2:gr
            y=[y; ygr{i,1}];
            yfit=[yfit; yfitgr{i,1}];
            end
            %DSS e TSS dei gruppi   
            for j=1:pe
            DSSgr=DSS{1,j};
            TSSgr=TSS{1,j};
                for i=2:gr
                DSSgr=DSSgr+DSS{i,j};
                TSSgr=TSSgr+TSS{i,j};
                end   
            R2(1,j)=DSSgr./TSSgr
            %R2 complessiva (quadrato di r)
            TSStutti=sum((y(:,1)-mean(y(:,1))).^2);
            RSStutti=sum((y(:,1)-yfit(:,1)).^2);
            DSStutti=sum((yfit(:,1)-mean(y(:,1))).^2);
            R2overall(1,j)=1-RSStutti/TSStutti
            %hold on
            %RMSE:
            RMSE(1,j)=(sum((y(:,1)-yfit(:,1)).^2)/(size(y,1))).^0.5
            end
            %hold off
     end
case 'pcrquadr' 
     if iscell(envir)==0
        [~,pe]=size(envir);
        y=envir;                   % Response (SST annual)
        X=mod;              % predittori
        [~,pm]=size(X);
        [PCALoadings,PCAScores,PCAVar] = pca(X,'Economy',false);
        figure
        plot(1:pm,(100*cumsum(PCAVar)/sum(PCAVar)),'-bo');
        hold on
        plot(1:pm,(100*(PCAVar)/sum(PCAVar)),'-r*');
        xlabel('Number of components');
        ylabel('Percent Variance Explained');
        hold off
        %dal grafico valutare i componenti
        prompt = 'How many components?';
        ncomp = input(prompt)   
        %procede con ncomp componenti, se pe  1 non ripete
            for j=1:pe           
            mdlPCR=fitlm(PCAScores(:,1:ncomp),y(:,j),'purequadratic');
            %nota: in table2array(mdl.Coefficients(2:end,1)) si trovano i coefficenti
            %come generati da regress (il primo elemento  l'intercetta)
            %i comandi seguenti servono a rappresentare il modello in termini di
            %variabili originali e non centrate.
            %come alternativa, si possono fattorizzare le osservazioni delle carote:
            %(daticarota-mediacolonnedatabase)*PCAloadings
            betap{1,j} = PCALoadings(:,[1:ncomp 1:ncomp])*table2array(mdlPCR.Coefficients(2:end,1));
            betap{1,j} = [mean(y(:,j)) - mean(X)*betap{1,j}; betap{1,j}];
            yfit(:,j)=[ones(size(X,1),1) X]*betap{1,j};
            beta(:,j)=betap{1,j};
            R2{1,j}=mdlPCR.Rsquared
            %RMSO
            RMSE(1,j)=(sum((y(:,j)-yfit(:,j)).^2)/(size(y(:,j),1)))^0.5
            %misurati vs stimati
            end
            R2overall=[];
 %se invece ci sono gruppi
    elseif iscell(envir)==1      
        [gr,~]=size(envir)
        [~,pe]=size(envir{1});
        %ripete per tutti i gr gruppi
        for i=1:gr
            %GRUPPO
            ygr{i,1}=envir{i,1}
            Xgr{i,1}=mod{i,1};              % predittori
            [~,pm]=size(Xgr{i,1});
            [PCALoadings,PCAScores,PCAVar] = pca(Xgr{i,1},'Economy',false);
            figure
            plot(1:pm,(100*cumsum(PCAVar)/sum(PCAVar)),'-bo');
            hold on
            plot(1:pm,(100*(PCAVar)/sum(PCAVar)),'-r*');
            xlabel('Number of components');
            ylabel('Percent Variance Explained');
            hold off
            %dal grafico valutare i componenti
            prompt = 'How many components?';
            ncomp=input(prompt)   
            %procede con ncomp componenti per tutti i pe (potrebbe
            %essere anche 1)
            for j=1:pe           
                mdlPCR=fitlm(PCAScores(:,1:ncomp),ygr{i,1}(:,j),'linear');
                betap{i,j} = PCALoadings(:,1:ncomp)*table2array(mdlPCR.Coefficients(2:end,1));
                betap{i,j} = [mean(ygr{i,1}(:,j)) - mean(Xgr{i,1})*betap{i,j}; betap{i,j}];
                yfitgr{i,1}(:,j)=[ones(size(Xgr{i,1},1),1) Xgr{i,1}]*betap{i,j};
                beta{i,1}(:,j)=betap{i,j}
                TSS{i,j} = sum((ygr{i,1}(:,j)-mean(ygr{i,1}(:,j))).^2);
                RSS{i,j} = sum((ygr{i,1}(:,j)-yfitgr{i,1}(:,j)).^2);
                DSS{i,j} = sum((yfitgr{i,1}(:,j)-mean(ygr{i,1}(:,j))).^2);
                DSS{i,j}./TSS{i,j}
                R2gr{i,j}=1-RSS{i,j}/TSS{i,j}
            end
        end
            %Rsquared_complessivo dei gruppi (y sono i dati ambientali)
            %se sono pi variabili, y{1}  una matrice
            %Se c' una sola variabile non ripete
%           figure
            y=ygr{1,1};
            yfit=yfitgr{1,1};
            for i=2:gr
            y=[y; ygr{i,1}];
            yfit=[yfit; yfitgr{i,1}];
            end    
            %DSS e TSS dei gruppi   
            for j=1:pe
            DSSgr=DSS{1,j};
            TSSgr=TSS{1,j};
                for i=2:gr
                DSSgr=DSSgr+DSS{i,j};
                TSSgr=TSSgr+TSS{i,j};
                end   
            R2(1,j)=DSSgr./TSSgr
            %R2 complessiva (quadrato di r)
            TSStutti=sum((y(:,1)-mean(y(:,1))).^2);
            RSStutti=sum((y(:,1)-yfit(:,1)).^2);
            DSStutti=sum((yfit(:,1)-mean(y(:,1))).^2);
            R2overall(1,j)=1-RSStutti/TSStutti
                %RMSEpooled:
            RMSE(1,j)=(sum((y(:,1)-yfit(:,1)).^2)/(size(y,1))).^0.5
            end
       end
otherwise
error('metodo non riconosciuto')
end
%___%::::::::::::::::::::::::::::::::________________________da qui si passa al fossile
if iscell(envir)==0
    %ricostruzione per il fossile si pu fare senza
    %analisi discriminante 
    ricfit=[ones(size(fossil,1),1) fossil(:,2:end)]*beta;
    ricfit=[fossil(:,1) ricfit];
%    figure
%    plot(ricfit(:,1),ricfit(:,2),'r')
    %ricostruzione per il fossile con
    %analisi discriminante 
elseif iscell(envir)==1 
    %crea la matrice per inserire l'appartenenza dei gruppi delle
    %associazioni moderne (1,2,etc)
    [n(1,1),~]=size(mod{1,1});
    for i=2:gr
    [n(i,1),~]=size(mod{i,1});
    end
    gruppiperclassify=zeros(sum(n),1);
    gruppiperclassify(1:n(1,1),1)=1;
    cums=cumsum(n);
    for i=2:gr
    gruppiperclassify(cums(i-1,1)+1:cums(i,1),1)=i;
    end
    %caso con gruppi ma stesse subcomposizioni e senza varargin{1},
    %assembla il moderno per l'analisi discriminante da mod
        if nargin==7
        modxdiscr=mod{1,1};
            for i=2:gr
            modxdiscr=[modxdiscr mod{i,1}];
            end
        %usa il fossil 
        fossilxdiscr=fossil(:,2:end);
    %caso con gruppi e subcomposizioni diverse (con varargin{1}),
        elseif nargin==8            
        modxdiscr=varargin{1}{1,1};
        %per fossil usa varargin{1}{2,1} 
        fossilxdiscr=varargin{1}{2,1};
        end
          %qui se i dati (fossili e moderni) sono ilr procede, se invece sono clr, li
          %trasfoma in ilr (che servono per la classificazione)
          switch tipodati
          case 'daticlr'
              modxdiscr=clrinversa(modxdiscr,100);
              fossilxdiscr=clrinversa(fossilxdiscr,100);
              modxdiscr=ilprogr(modxdiscr);
              fossilxdiscr=ilprogr(fossilxdiscr);
          case 'datiilr'
          otherwise
          error('tipodati can be daticlr or datiilr')    
          end
          %procede con l'analisi discriminante
          [classfossil,err,POSTERIOR]=classify(fossilxdiscr,modxdiscr,gruppiperclassify);
          %sulla base della classificazione, procede alle stime per le
          %osservazioni fossili (una alla volta sulla base della classificazione)

    %caso con gruppi ma stesse subcomposizioni e senza varargin{1},
    %usa fossil
        if nargin==7
           [nf,~]=size(fossil);
           for j=1:pe
              for i=1:nf
              ricfit(i,j)=[1 fossil(i,2:end)]*beta{classfossil(i,1),j};
              end
           end
           ricfit=[fossil(:,1) ricfit];
    %caso con gruppi e subcomposizioni diverse (con varargin{1})
    %usa fossil che per  cell array,
        elseif nargin==8
           [nf,~]=size(fossilxdiscr);
           for j=1:pe           
               for i=1:nf
 %                  ricfit(i,j)=[1 fossil{classfossil(i,1),1}(i,2:end)]*beta{classfossil(i,1),j};
                   ricfit(i,j)=[1 fossil{classfossil(i,1),1}(i,2:end)]*beta{classfossil(i,1),1}(:,j);
               end
           end
        end
     ricfit=[fossil{1,1}(:,1) ricfit];
end
validation.environment=y;
validation.environmentfitted=yfit;
validation.RMSE=RMSE
validation.R2=R2
validation.R2overall=R2overall
    for i=1:pe
    validation.mdl{1,i}=fitlm(y(:,i),yfit(:,i))
    figure
    plot(validation.mdl{1,i})
%    figure
%    plot(y(:,i),yfit(:,i),'.')
    title(Mtext(1,i+1))
    xlabel('measured');
    ylabel('estimated');
    end
    
switch Zscores
    case 'yes'
        for i=1:pe
        yy(:,i)=(y(:,i).*deviazstd(1,i))+medie(1,i);
        yyfit(:,i)=(yfit(:,i).*deviazstd(1,i))+medie(1,i);
        ricfit(:,i+1)=(ricfit(:,i+1).*deviazstd(1,i))+medie(1,i);
        RMSE(1,i)=(RMSE(1,i).*deviazstd(1,i))+medie(1,i);
        end
    case 'no'
end

switch grafici
     case 'insieme'
     figure
     plot(ricfit(:,1),ricfit(:,2:end));
     case 'subplots'
     plotricreg(ricfit,'vert',Mtext,Mtext(1,1),RMSE,[])
     otherwise
     error('metodo non riconosciuto!')
     end
     hold off
end

        
 