%this code is provided as-is without any warranty or indemnification of any kind. 
%for any problem please contact me: e-mail adress: valedido@unina.it
%if you are using it please cite:
%Di Donato,V., Jamka. J., Martn-Fernndez J.A. 2020 - Compositional regression-based methods for SST reconstruction
%Alpine and Mediterranean Quaternary,... 

%Plots the best analogs of a fossil composition on an Earth map
%the ranking of the analogs is shown on the left of the markers
%
%HOW TO USE
%obs                            is the running number of the fossil composition in F 
%latAn e longAn                 are lat and long of the best analogs
%                               obtained from the function mat
%analogs                        define how many analogs are plotted.  
%quality                        can be 'low' (black & white coast map); 'world'  (with rivers, lakes etc);
%                               'colorlow' (colored map with topography,
%                               but low resolution); 'etopomap' (high quality color map with etopo data: N.B. it requires the 
%                               file 'etopo1_ice_c_i2.bin'!))
%limiti                         is a 4 element row with latitudinal (-90 to 90) and longitudinal(da -180 a 180)
%                               limits
%ID                             can be: 'si' o 'no'. With 'si' insert the
%                               ranking of analogs (1,2,3,,,,), with 'no'
%                               only circles
%varargin{1}        IX          (facultative) refer to the IX matrix obtained from MAT, and insert, 
%                                   on the right of each analog marker, its running number
%varargin{2}, varargin{3} and  varargin{4} allow to insert, respectively, the latitude, the longitude, and the
%name of the core. A marker will appear in the map
%
%example1: mapanalog(i,latAn,longAn,7,'high',[0 90 -140 90],'no',IX,39.77,17.05,'TEA C6');
%esempio2: mapanalog(20,latAnwe,longAnwe,7,'low',[0 90 -140 90],'no',IX,modv5jef(20,2),modv5jef(20,3))
%esempio3: mapanalog(20,latAnwe,longAnwe,7,'low',[-60 90 -120 45],'si',modv5jef(20,2),modv5jef(20,3))
function mapanalog(obs,latAn,longAn,analogs,quality,limiti,ID,varargin)
%sceglie tra bianco e nero e colore
switch quality
    case 'low'
    %carica il file con la linea di costa
        load coast
        %crea una mappa con la proiezione di mercatore
           figure; axesm('MapProjection','miller','Frame','on','FLineWidth',1,'Origin',[0 0 0],'MapLatLimit',[limiti(1,1:2)],'MapLonLimit',[limiti(1,3:4)],...
    'Grid','on','MLineLocation',30,'PLineLocation',15,'MeridianLabel','on','ParallelLabel','on','FontSize',8) 
    %borda la figura
    %framem('on')
    box('off')
    %inserisce meridiani e paralleli in nero
    plotm(lat,long,'k')
    
    case 'world'
    %carica il file con la linea di costa
    ax = worldmap(limiti(1,1:2),limiti(1,3:4));
    load coastlines
    plotm(coastlat,coastlon)
    land = shaperead('landareas', 'UseGeoCoords', true);
    geoshow(ax, land, 'FaceColor', [0.5 0.7 0.5])
    lakes = shaperead('worldlakes', 'UseGeoCoords', true);
    geoshow(lakes, 'FaceColor', 'blue')
    rivers = shaperead('worldrivers', 'UseGeoCoords', true);
    geoshow(rivers, 'Color', 'blue')
    
    case 'colorlow'     
    %carica il file con la topografia   
    load topo 
    %PROPRIETA': 1. proiezione 2. frame 3. origine (lat long angle), 4.limita i paralleli
    %5. limita i meridiani 6. mostra griglia 7. spaziatura meridiani 8. spaziatura
    %paralleli 9.e 10. etichette 11.dimensioni
    figure; axesm('MapProjection','miller','Frame','on','FLineWidth',1,'Origin',[0 0 0],'MapLatLimit',limiti(1,1:2),'MapLonLimit',limiti(1,3:4),...
    'Grid','on','MLineLocation',30,'PLineLocation',15,'MeridianLabel','on','ParallelLabel','on','FontSize',8) 
    meshm(topo,topolegend); demcmap(topo)
    %borda la figura 
    %framem  
    box('off')
 
    case 'etopomap'
    samplefactor = 10;
    [Zet, refvec] = etopo('etopo1_ice_c_i2.bin', samplefactor);
    worldmap([limiti(1,1:2)],[limiti(1,3:4)])
    geoshow(Zet, refvec, 'DisplayType', 'texturemap');
    demcmap(Zet, 256);
    geoshow('landareas.shp', 'FaceColor', 'none', ...
    'EdgeColor', 'black');
    clear refvec
    clear Zet
    hold on 
    
    otherwise error('parametro non riconosciuto');
end

%aggiunge elementi nella stessa mappa
hold on 
[jj kk]=size(latAn);
if analogs>kk ;
    error('pi analoghi di quelli esistenti!')
end
%plotta gli analoghi (vettori colonna!) con cerchi blu, pieni giallo
%plotm(latAn(obs,101:200),longAn(obs,101:200),'oc','MarkerFaceColor','c','MarkerSize',2);
%hold on
%plotm(latAn(obs,51:100),longAn(obs,51:100),'ob','MarkerFaceColor','b','MarkerSize',2);
%plotm(latAn(obs,21:50),longAn(obs,21:50),'og','MarkerFaceColor','g','MarkerSize',2);
%plotm(latAn(obs,11:20),longAn(obs,11:20),'om','MarkerFaceColor','m','MarkerSize',2);
%plotm(latAn(obs,7:10),longAn(obs,7:10),'or','MarkerFaceColor','r','MarkerSize',2);
plotm(latAn(obs,1:analogs),longAn(obs,1:analogs),'oy','MarkerFaceColor','y','MarkerSize',2);

switch ID
    case 'si'
        %inserisce il ranking dell'analogo
        for h=1:analogs;
        H=h;
        %converte in stringa ogni elemento progressivo 1,2,...,an. Inserisce le
        %stringhe nella mappa
        X=num2str(H);
        textm(latAn(obs,h),longAn(obs,h)+0.5,X,'Fontsize',7,'color','k');
        end
    case 'no'
end
%Se non c'e IX si ferma 
if nargin==7
hold off
%Se non c'e solo IX
elseif nargin==8
    IX=varargin{1};
%Se c' IX inserisce anche il running number dell'analogo
    %richiama dalla riga obs, per ogni h, il rispettivo running number
    for h=1:analogs;
    J=IX(obs,h);
    %converte in stringa J per ogni h=1:analogs
    W=num2str(J);    
    textm(latAn(obs,h)+0.5,longAn(obs,h),W,'Fontsize',7,'color','g');
    end
    holf off
%Se non c'e IX ma ci sono corelat,corelong,
elseif nargin==9
    corelat=varargin{1};
    corelong=varargin{2};
    %mette un asterisco rosso nella mappa per identificare la carota
    plotm(corelat,corelong,'*r','MarkerSize',8)
    %inserisce il nome 'sigla' alle latitudini e longitudini specificate
    %textm(IX+1,corelat,corelong,'Fontsize',10,'color','r'); 
    hold off
%Se c'e IX e ci sono corelat,corelong,
elseif nargin==10
    IX=varargin{1};
    corelat=varargin{2};
    corelong=varargin{3};
    %Se c' IX inserisce anche il running number dell'analogo
    %richiama dalla riga obs, per ogni h, il rispettivo running number
    for h=1:analogs;
    J=IX(obs,h);
    %converte in stringa J per ogni h=1:analogs
    W=num2str(J);    
    textm(latAn(obs,h)+0.5,longAn(obs,h),W,'Fontsize',7,'color','g');
    end
    %mette un asterisco rosso nella mappa per identificare la carota
    plotm(corelat,corelong,'*r','MarkerSize',8)
    %inserisce il nome 'sigla' alle latitudini e longitudini specificate
    %textm(IX+1,corelat,corelong,'Fontsize',10,'color','r'); 
    hold off
   %Se c'e IX e ci sono corelat,corelong, sigla
 elseif nargin==11
     IX=varargin{1};
    corelat=varargin{2};
    corelong=varargin{3};
    sigla=varargin{4};
 %Se c' IX inserisce anche il running number dell'analogo
    %richiama dalla riga obs, per ogni h, il rispettivo running number
    for h=1:analogs;
    J=IX(obs,h);
    %converte in stringa J per ogni h=1:analogs
    W=num2str(J);    
    textm(latAn(obs,h)+0.75,longAn(obs,h),W,'Fontsize',6,'color','g');
    end
 %inserisce la carota nel plot
    plotm(corelat,corelong,'*r','MarkerSize',8)
%inserisce il nome 'sigla' alle latitudini e longitudini specificate
    textm(corelat+1,corelong,sigla,'Fontsize',10,'color','r'); 
    hold off
end
%DA RIFINIRE: NELLA VERSIONE LOW PROVARE A FARE TUTTO BIANCO E
%NERO
%Volendo si pu FARE UNA VERSIONE AUTOMATICA CON TUTTE LE OSSERVAZIONI

%N.B: worldhi non c' pi in matlab 7
%N.B.B. si possono usare corelat e corelong richiamandoli da una matrice
%n.b.b.b. bisogna ancora lavorare su worldmapold 

