%this code is provided as-is without any warranty or indemnification of any kind. 
%for any problem please contact me: e-mail adress: valedido@unina.it
%if you are using it please cite:
%Di Donato,V., Jamka. J., Martn-Fernndez J.A. 2020 - Compositional regression-based methods for SST reconstruction
%Alpine and Mediterranean Quaternary,... 

%Modern analogues technique computed from a matrix M of modern assemblages and a matrix F of "fossil assemblages  
%How to use: input 
%M   	is the matrix which includes the dataset with Modern observations; The first 3 columns of M contain 
%       respectively: id. number, latitude, longitude; the following ones the environmental parameters and, to follow, the abundance of taxa
%F   	is the matrix of "fossil" assemblages, the first columm includes either a running number 
%       (or cm or age for depths into a core). Be careful that the number and the order of taxa is the same in both F and M!
%param	the number of environmental parameters included in M
%dist  	the adopted distance measure: it can be 'euclidean','squared
%         euclidean','sqchord' (squared chord distance),'Mahalanobis', 'costheta'
%an     indicates the number of modern analogues from which the palaeoestimate is computed; It 
%       can be either a number (the same for all) or a column vector with size(an)=[number of rows of f,1].     
%media  'un' indicates a simple mean estimate. 'we' indicates that the estimates are weigthed 
%       on the inverse of the distance (or on the similarity for costheta).
%tipodati   the kind of data included for the assemblages. It is adopted 
%           for the computation of the suspicious non analogs. Can be: 'ilr', 
%           'clr', 'reali' (real numbers), composizionali' (i.e. percentages)
%regolaPsuN     indicates how many analogs are considered to evaluate the atipicality of fossils 
%               assemblages. Insert tutti as default. With '10x' the computation is 
%               referred to the atypicality respect to 10 times the number of taxa (the closest). With '5x' (5 times the number of taxa
%varargin{1} by adding not1 the function excludes the first analogue: it is useful for leave-one-out verification
%output:
%by typing. 
%[Ds,IX,latAn,longAn,ric,meanD,distanzalimite,noanalogs]=mat(F,M,param,dist,an,media,tipodati,oneminusalpha,regolaPsuN) 
%the output returns:
%Ds       		Analogs distances ordered in ascending order
%IX       		running number of the analogs as they appear in Ds
%latAn    		latitude of modern analogs as they appear in Ds
%longAn   		longitude of modern analogs as they appear in Ds
%ric      		palaeoenvironmental estimates with the corresponding std for each parameter (the first column is the first column of F)
%meanD    		the mean of the distances between fossil and the selected analogs
%distanzalimite 	is the threshold distance obtained within the modern dataset with the selected confidence interval
%noanalogs 	     a two column matrix: the first column report the squared mahalanobis distances of 
%                fossil assemblages from the center of the modern dataset, the second column is made of 0 and 1: 
%                1 correspond to a significant value (a suspicious non analog assemblages)
%application example:
%[Ds,IX,latAn,longAn,ric,meanD,distanzalimite,noanalogs]=mat(F,M,4,euclidean,6,we,clr,0.999,tutti) 
function [Ds,IX,latAn,longAn,ric,meanD,distanzalimite,noanalogs]=mat(F,M,param,dist,an,media,tipodati,oneminusalpha,regolaPsuN,varargin) %F per il fossile, M per il moderno
%dimensioni delle composizioni in F e M. 
[f v]=size(F(:,2:end));
[m w]=size(M(:,param+4:end));
%controlla se le variabili sono le stesse
if v~=w;
    error('il numero di variabili non  uguale!')
end
%INSERIRE ALTRE DISTANZE!

%crea la matrice per il calcolo delle distanze tra righe di F e righe di M 
D=zeros(f,m);
%calcola la matrice delle distanze utilizzando diverse distanze
switch dist
    case 'costheta'
        for i=1:f 
            for j=1:m
            D(i,j)=(sum((F(i,2:end)).*(M(j,(4+param):end))))/(sqrt(sum((F(i,2:end).^2)).*(sum(M(j,(4+param):end).^2))));
            end    
        end
    case 'euclidean'
        for i=1:f 
            for j=1:m
            diff=(F(i,2:end)-M(j,(4+param):end));
            sq=diff.^2;
            D(i,j)=sqrt(sum(sq')');
            end    
        end
    case 'squared euclidean' 
        for i=1:f
            for j=1:m
            diff=(F(i,2:end)-M(j,(4+param):end));
            sq=diff.^2;
            D(i,j)=sum(sq')';
            end    
        end
    case 'mahalanobis'
            iM=[M(:,(4+param):end)];
            IM=pinv(cov(iM));
            for i=1:f
                 for j=1:m
                 D(i,j)=sqrt((F(i,2:end)-M(j,(4+param):end))*IM*(F(i,2:end)-M(j,(4+param):end))'); 
                 end
            end        
    case 'sqchord'
        for i=1:f
            for j=1:m
            diff=(sqrt(F(i,2:end)))-(sqrt(M(j,(4+param):end)));  
            sq=diff.^2;
            D(i,j)=sum(sq')';
            end
        end
    otherwise
        error('misura di distanza non riconosciuta')
   end      
%ordina in senso ascendente ciascuna riga (Ds) e mostra la permutazione effettuata (IX)
[Ds,IX] = sort(D,2);
%REM seleziona le prime 'an' colonne di IX, in relazione al numero di analoghi
%REM  X=IX(:,(1:an));

%poich 'costheta'  un coefficiente di similarit, ruota da dx a sn le matrici Ds e IX  
switch dist
    case 'costheta'
    Ds=fliplr(Ds);
    IX=fliplr(IX);
otherwise 
end

%verifica se ci sono campioni atipici
switch tipodati
    case 'clr'
    %calcola l'inversa e poi la ilr dei dati moderni e fossili
    clrinvM=clrinversa(M(:,param+4:end),100);
    clrinvF=clrinversa(F(:,2:end),100);
    [ilrM,PSIM]=ilrprogr2(clrinvM);
    [ilrF,PSIF]=ilrprogr2(clrinvF);
    vv=v-1;
    case 'ilr'
    ilrM=M(:,param+4:end);
    ilrF=F(:,2:end);
    vv=v;
    case'reali'
    ilrM=M(:,param+4:end);
    ilrF=F(:,2:end);
    vv=v;
    case 'composizionali'
    %casomai non fossero gi chiusi, trasforma i dati in proporzioni
        Fprov=zeros(n,v);
        Pprov=zeros(M,w);
        for i=1:f
            if sum(F(i,2:end))~=1
            Fprov(i,:)=F(i,2:end)/sum(F(i,2:end));
            end
        end
        for i=1:m
            if sum(M(:,param+4:end))~=1
            Mprov(i,:)=M(i,param+4:end)/sum(M(i,param+4:end));
            end
        end
        %calcola la ilr dei dati moderni e fossili
    [ilrM]=ilrprogr2(Mprov);
    [ilrF]=ilrprogr2(Fprov);
    vv=v-1;
    otherwise
        error('tipo dati non riconosciuto')
end
%diStanza limite: N.B. in Hair et al (Multivariate data analysis
%viene suggerito un valore molto conservativo, ossia un livello 0.001
%che equivarrebbe a 0.999
distanzalimite=chi2inv(oneminusalpha,vv); %0.996 %0.995
%distanza mahal al quadrato tra ogni campione rispetto al database moderno
switch regolaPsuN
    case 'tutti'
    distanzemahal=mahal(ilrF,ilrM);
    case '10x'
    for i=1:f
    distanzemahal(i,:)=mahal(ilrF(i,:),ilrM(IX(i,1:(10*vv)),:));
    end
    case '5x'
    for i=1:f
    distanzemahal(i,:)=mahal(ilrF(i,:),ilrM(IX(i,1:(5*vv)),:));
    end
    otherwise
    error('regolaPsun pu essere tutti, 5x o 10x')
end
%crea una colonna di 0 e 1: 1 sono possibili non analoghi
noanalogs=zeros(f,1);
for i=1:f
    if distanzemahal(i,1)>distanzalimite
    noanalogs(i,1)=1;
    else
    noanalogs(i,1)=0;    
    end
end
noanalogs=[distanzemahal noanalogs];
%CREA i grafici delle distanze per gli analoghi selezionati
%DISATTIVATO! PER ATTIVARE RIMUOVERE GLI '%REM'
%REM for i=1:f
%REM figure(i)
%REM plot (Ds(i,1:an),'-*')
%REM end
%Genera le matrici per inserire i valori ambientali degli analoghi
%ordinati
for t=1:param
par{t}=zeros(f,m);
end
%calcola le matrici, ogni par{t}  per uno dei parametri. 
for t=1:param
    for i=1:f
        for k=1:m
        par{t}(i,k)=(M((IX(i,k)),3+t));
        end
    end
end
%Coordinate geografiche degli analoghi ordinati in senso ascendente per
%righe 
latAn=zeros(f,m);
longAn=zeros(f,m);
for i=1:f
    for k=1:m
latAn(i,k)=(M((IX(i,k)),2));
longAn(i,k)=M(IX(i,k),3);
    end
end
%diversifica il numero di analoghi
if size(an)==[1,1]
    ana=ones(f,1);
    for i=1:f
        ana(i,1)=an;
    end
elseif size(an)==[f,1]
    ana=an;
elseif size(an)~=[f 1] || [1 1]
    error('an deve essere un vettore colonna con size(an)=[f,1]!')
end
%stime paleoambientali, in relazione a 'we' o 'un' e 'not1'. In tutto ci
%sono 4 casi
EST=zeros(f,param);
STD=zeros(f,param);
if nargin<10  %set default to consider all the first analogs     
    switch media
          case 'un'
                for i=1:f
                    for t=1:param   
                    EST(i,t)=(sum(par{t}(i,1:ana(i,1))))./(ana(i,1));
                    STD(i,t)=std(par{t}(i,1:ana(i,1)));
                    end
                end     
          case 'we'
                switch dist
                    case 'costheta'
                        for i=1:f
                            for t=1:param
                            EST(i,t)=mediapond(par{t}(i,1:ana(i,1)),Ds(i,1:ana(i,1)));
                            STD(i,t)=std(par{t}(i,1:ana(i,1)));
                            end 
                        end
                    otherwise
                        for i=1:f
                            for t=1:param
                            EST(i,t)=mediapondinv(par{t}(i,1:ana(i,1)),Ds(i,1:ana(i,1)));
                            STD(i,t)=std(par{t}(i,1:ana(i,1)));
                            end 
                        end
                end
          case 'we2'   
                switch dist
                    case 'costheta'
                    for i=1:f
                        for t=1:param 
                        EST(i,t)=mediapond(par{t}(i,1:ana(i,1)),(Ds(i,1:ana(i,1))).^2);
                        STD(i,t)=std(par{t}(i,1:ana(i,1)));
                        end 
                    end                  
                    otherwise    
                    for i=1:f
                        for t=1:param 
                        EST(i,t)=mediapondinv(par{t}(i,1:ana(i,1)),(Ds(i,1:ana(i,1))).^2);
                        STD(i,t)=std(par{t}(i,1:ana(i,1)));
                        end 
                    end    
                end
          otherwise
          error('media pu essere un, we o we2')
          end
%qui viene escluso il primo analogo (se stesso se si comparano moderni con
  %moderni)
elseif nargin==10
    if varargin{1}=='not1'
    switch media
          case 'un'
                for i=1:f  
                    for t=1:param
                    EST(i,t)=(sum(par{t}(i,2:ana(i,1)+1)))./(ana(i,1));
                    STD(i,t)=std(par{t}(i,2:ana(i,1)+1));
                    end
                end    
          case 'we'
                switch dist
                    case 'costheta'
                      for i=1:f
                            for t=1:param
                            EST(i,t)=mediapond(par{t}(i,2:ana(i,1)+1),Ds(i,2:ana(i,1)+1));
                            STD(i,t)=std(par{t}(i,2:ana(i,1)+1));
                            end
                      end
                    otherwise
                      for i=1:f
                            for t=1:param
                            EST(i,t)=mediapondinv(par{t}(i,2:ana(i,1)+1),Ds(i,2:ana(i,1)+1));
                            STD(i,t)=std(par{t}(i,2:ana(i,1)+1));
                            end
                      end
                end
          case 'we2'
                switch dist
                    case 'costheta'
                        for i=1:f
                            for t=1:param
                            EST(i,t)=mediapond(par{t}(i,2:ana(i,1)+1),(Ds(i,2:ana(i,1)+1)).^2);
                            STD(i,t)=std(par{t}(i,2:ana(i,1)+1));
                            end
                        end
                    otherwise
                        for i=1:f
                            for t=1:param
                            EST(i,t)=mediapondinv(par{t}(i,2:ana(i,1)+1),(Ds(i,2:ana(i,1)+1)).^2);
                            STD(i,t)=std(par{t}(i,2:ana(i,1)+1));
                            end
                        end
                end
            end
    end
end
%unisce le matrici con stime e relativa std
TMP=[EST STD];
  %crea una matrice Z di permutazione per riordinare TMP, in modo da avere, per ogni
%parametro, media e std in successione
u=2*param;
zz=zeros(1,u);
[ss,tt]=size(zz);
%si crea prima un vettore zz: per le colonne dispari, valori crescenti per unit, per le colonne pari
%valori crescenti per unit a partire da param+1
for q=1:tt
    if rem(q,2)==1
    zz(q)=(q+1)/2;
    else
    zz(q)=(q+u)/2;
    end
end
%da zz si crea la matrice Z
Z=zeros(u);
%si inseriscono valori 1 solo negli elementi q,zz(q) 
for q=1:u
Z(q,zz(q))=1;
end
%I RISULTATI
ric=[F(:,1) TMP*Z'];
%calcola, per ogni campione, la media delle distanze degli analoghi
%selezionati.
meanD=zeros(f,1);
for i=1:f
    if nargin<10 & size(an)==[1,1]
    meanD(i,1)=sum(Ds(i,1:an))./an;
    elseif nargin<10 & size(an)==[f,1]
    meanD(i,1)=sum(Ds(i,1:an(i,1)))./an(i,1);
    elseif nargin==10 & size(an)==[1,1]
    meanD(i,1)=sum(Ds(i,2:an+1))./an;
    elseif nargin==10 & size(an)==[f,1] 
    meanD(i,1)=sum(Ds(i,2:an(i,1)+1))./(an(i,1)+1);
    end
end
meanD=[F(:,1) meanD];

%DA rifinire: si porebbe impostare direttamente l'intervallo di analoghi da
%considerare (ad es. 1:10, 2:11 etc)
end


%------------------------------------------------------------------------------------
function m=mediapondinv(x,y)
%   Average of the values of vector x weighted on the inverse of the values of a vector y
% N.B. In order to avoid a Warning: 'Divide by zero', the elements of y==0
% are substituded with a very small value (0.00001) 
[a,b]=size(x);
[c,d]=size(y);
if b~=d
    error('le dimensioni non sono uguali')
end
for k=1:d
    if y(k)==0
        y(k)=0.00001;
    end
end    
m = (sum(x.*(1./y)))/(sum(1./y));
end

%---------------------------------------------------------------------------
function m = mediapond(x,y)
%   Average of the values of vector x weighted on the values of a vector y
  m = (sum(x.*y))/(sum(y))
end
