%this code is provided as-is without any warranty or indemnification of any kind. 
%for any problem please contact me: e-mail adress: valedido@unina.it
%if you are using it please cite:
%Di Donato,V., Jamka. J., Martn-Fernndez J.A. 2020 - Compositional regression-based methods for SST reconstruction
%Alpine and Mediterranean Quaternary,...

%plots the palaeoenvironmental reconstructions computed with the mat
%function
%
%ric             is the output of mat function
%rotazione       can be 'oriz' or 'vert': 'oriz' makes separate graphs with
%                the age (depht) as x axis; 'vert' makes joined subplot graphs, with
%                the age (depht) as y axis
%errore_si_o_no can be: 'barre_si' or 'barre_no'. 'barre_si' adds error
%                bars to the graphs
%Mtext           is the cell array with the column headers of the M file (as in the mat function)
%depthlabel      is the label for the y axis (usually 'cm' or 'age')
%oggi (optional) includes the modern data from which compute the anomalies
%                (it is a string with the values of the M environmental variables)
%oggizero        (to be used with oggi), can be: 'si' or 'no'. With 'si'
%                the anomalies for the topmost sample are forced to 0 (to be only adopted when )
%                the topmost sample is modern), with 'no' they are simply computed 
%                as differences from "oggi" values. 
%
%
%example:        plotric(ric,'vert','barre_si',Mtext,Ftext,oggi,oggizero)
function plotric(ric,rotazione,errore_si_o_no,Mtext,depthlabel,oggi,oggizero)
%rotazione rende il grafico in orizzontale o in verticale
%%oggi, facoltativo, contiene i dati moderni da cui calcolare le anomalie
%oggizero, da inserire se c' oggi, pu essere 'si' o 'no'. se  si il top va a
%0 per tutti i valori se  no, oggi va inserita come []
[L,P]=size(ric);
%crea una copia di ric con le anomalie invece dei valori assoluti
%se c' "oggi" lo usa per calcolare le anomalie
if nargin==7
    ric_anomalie=ric;
    switch oggizero
    %se 'si' impone che la prima riga di ric_anomalie sia 0 per tutti i valori
    case 'si'
        ric_anomalie(1,2:end)=0;
        for y=1:(P-1)/2
            for i=2:L
            ric_anomalie(i,2*y)=ric_anomalie(i,2*y)-oggi(1,y);
            end
        end
    %se 'no' semplicemente sottrae i valori di ric da quelli di oggi. Pu
    %servire se la successione non arriva a oggi
    case 'no'
        for y=1:(P-1)/2
            ric_anomalie(:,2*y)=ric_anomalie(:,2*y)-oggi(1,y);
        end
    end
elseif nargin==5
    ric_anomalie=ric;
    for y=1:(P-1)/2
    ric_anomalie(:,2*y)=ric_anomalie(:,2*y)-ric(1,2*y);
    end
end
    
%minimo dei minimi
minmin=min(min(ric(:,2:2:P-1)));
minmina=min(min(ric_anomalie(:,2:2:P-1)));
%massimo dei massimi
maxmax=max(max(ric(:,2:2:P-1)));
maxmaxa=max(max(ric_anomalie(:,2:2:P-1)));
%massimo delle std
maxstd=max(max(ric(:,3:2:P)));    
maxstda=max(max(ric_anomalie(:,3:2:P)));        
    %crea i grafici
    switch rotazione
    %grafici singoli
    case 'oriz'
        for y=1:(P-1)/2
        figure
        plot(ric(:,1),ric(:,2*y),'Linewidth',1.5);
        %invece di ric(1,1) prima c'era 0
        axis([ric(1,1) ceil(max(ric(:,1))) min(ric(:,2*y))-(2*max(ric(:,2*y+1))/2) max(ric(:,2*y))+(2*max(ric(:,2*y+1))/2)]);
            if nargin==5
            ylabel(Mtext(1,y+3));
            xlabel(depthlabel);
            end
        hold on
        switch errore_si_o_no
            case 'barre_si'
            errorbar(ric(:,1),ric(:,2*y),ric(:,2*y+1));
            case 'barre_no'
        end
        set(gca,'XTickMode','auto','XMinorTick','on')
    %else
    %set(gca,'xtick',[0:2:ceil(max(ric(:,1)))],'XMinorTick','on')
    %end    
        hold off
        end
        %grafico anomalie
        for y=1:(P-1)/2;
        figure;
        plot(ric_anomalie(:,1),ric_anomalie(:,2*y),'Linewidth',1.5)
        axis([ric(1,1) ceil(max(ric_anomalie(:,1))) min(ric_anomalie(:,2*y))-(2*max(ric_anomalie(:,2*y+1))/2) max(ric_anomalie(:,2*y))+(2*max(ric_anomalie(:,2*y+1))/2)]);
            if nargin==5
            ylabel(Mtext(1,y+3));
            xlabel(depthlabel);
            end
        hold on
        switch errore_si_o_no
            case 'barre_si'
            errorbar(ric_anomalie(:,1),ric_anomalie(:,2*y),ric_anomalie(:,2*y+1));
            case 'barre_no'
        end
        set(gca,'XTickMode','auto','XMinorTick','on')
        %else
        %set(gca,'xtick',[0:2:ceil(max(ric(:,1)))],'XMinorTick','on')
        %end    
        hold off
        end
    %grafici per subplot
    case 'vert'
    %definisce la successione di colori (fino a 6) rosso scuro rosso arancione blu
    %verde nero 
    rossoarabluver=[0.5 0 0;1 0 0; 1 0.4 0.2; 0 0 1; 0 1 0; 0 0 0; 0 0 0 ; 0 0 0; 0 0 0];
    figure
    %set(gcf,'PaperUnits','centimeters','PaperPositionMode','manual','PaperPosition', [0.1 0.1 ((P-1)/2)*6 14]);
    set(gcf,'DefaultAxesColorOrder',rossoarabluver(1,:))
    %crea il primo subplot
    %subplot(1,(P-1)/2,1)
    subplot('position',[0.1 0.1 0.1 0.8])
    switch errore_si_o_no
            case 'barre_si'
            errorbar(ric(:,1),ric(:,2),ric(:,2+1),'color',[0.6 0.6 0.6],'LineWidth',0.2)
        case 'barre_no'
    end
    hold on
    plot(ric(:,1),ric(:,2),'Linewidth',0.5);
%    axis([ric(1,1) ceil(max(ric(:,1))) ceil(minmin-maxstd/2)-1 ceil(maxmax+maxstd/2)]);
    axis([ric(1,1) ceil(max(ric(:,1))) -inf inf]);
        if nargin==5
        ylabel(Mtext(1,4));
        xlabel(depthlabel);
        end
    set(gca,'XTickMode','auto','XMinorTick','on','XAxisLocation','bottom','YAxisLocation','right','YMinorTick','on') %FontSize',5,'units','centimeters','position',[1 1 3 12])
    view([90 90])
    %subplot successivi, l'unica differenza  che non appare l'etichetta
    %dell'asse Y (age o cm)
        for y=2:(P-1)/2
        set(gcf,'DefaultAxesColorOrder',rossoarabluver(y,:))
        subplot('position',[0.1+(y-1)*0.15 0.1 0.1 0.8])
        switch errore_si_o_no
            case 'barre_si'
            errorbar(ric(:,1),ric(:,2*y),ric(:,2*y+1),'color',[0.6 0.6 0.6],'LineWidth',0.2)
            case 'barre_no'
        end
        hold on
        plot(ric(:,1),ric(:,2*y),'Linewidth',0.5);
%        axis([ric(1,1) ceil(max(ric(:,1))) ceil(minmin-maxstd/2) ceil(maxmax+maxstd/2)]);
        axis([ric(1,1) ceil(max(ric(:,1))) -inf inf]);
           if nargin==5
           ylabel(Mtext(1,y+3));
           end
           set(gca,'XTickMode','auto','XMinorTick','on','XAxisLocation','bottom','YAxisLocation','right','YMinorTick','on') %'FontSize',5,'units','centimeters','position',[(3.4*y)-2.4 1 3 12])
    %else
        %set(gca,'xtick',[0:2:ceil(max(ric(:,1)))],'XMinorTick','on')
        %end 
        %imposta le dimensioni della figura

%set(gcf,'PaperUnits','centimeters')
%set(gcf,'PaperPositionMode','manual');
%set(gcf,'PaperPosition',[0.1 0.1 (3*P)+2 12]);
hold off    
        %set(gcf,'PaperUnits','centimeters','PaperPositionMode','manual','PaperPosition', [0.1 0.1 ((P-1)/2)*4+2*((P-1)/2) 14]);
        view([90 90])
 set(gcf, 'units','centimeters','PaperPosition', [1 1 25 15]);
        end
%grafico anomalie
    figure
    %set(gcf,'PaperUnits','centimeters','PaperPositionMode','manual','PaperPosition', [0.1 0.1 ((P-1)/2)*6 14]);
    set(gcf,'DefaultAxesColorOrder',rossoarabluver(1,:))
    subplot(1,(P-1)/2,1)
    switch errore_si_o_no
            case 'barre_si'
    errorbar(ric_anomalie(:,1),ric_anomalie(:,2),ric_anomalie(:,2+1),'color',[0.6 0.6 0.6],'LineWidth',0.2);
        case 'barre_no'
    end 
    hold on;
    plot(ric_anomalie(:,1),ric_anomalie(:,2),'Linewidth',0.5);
%    axis([ric(1,1) ceil(max(ric(:,1))) ceil(minmina-maxstd/2) ceil(maxmaxa+maxstd/2)]);
    axis([ric(1,1) ceil(max(ric(:,1))) -inf inf]);
        if nargin==5
        ylabel(Mtext(1,4));
        xlabel(depthlabel);
        end
    set(gca,'XTickMode','auto','XMinorTick','on','XAxisLocation','bottom','YAxisLocation','right','YMinorTick','on') %FontSize',5,'units','centimeters','position',[1 1 3 12])
    view([90 90])
    %subplot successivi, l'unica differenza  che non appare l'etichetta
    %dell'asse Y (age o cm)
        for y=2:(P-1)/2;
        set(gcf,'DefaultAxesColorOrder',rossoarabluver(y,:))
        subplot(1,(P-1)/2,y)
        switch errore_si_o_no
            case 'barre_si'
        errorbar(ric_anomalie(:,1),ric_anomalie(:,2*y),ric_anomalie(:,2*y+1),'color',[0.6 0.6 0.6],'LineWidth',0.2);
            case 'barre_no'
        end
        hold on;
        plot(ric_anomalie(:,1),ric_anomalie(:,2*y),'Linewidth',0.5);
%        axis([ric(1,1) ceil(max(ric(:,1))) ceil(minmina-maxstd/2) ceil(maxmaxa+maxstd/2)]);
        axis([ric(1,1) ceil(max(ric(:,1))) -inf inf]);
            if nargin==5
            ylabel(Mtext(1,y+3));
            end
        set(gca,'XTickMode','auto','XMinorTick','on','XAxisLocation','bottom','YAxisLocation','right','YMinorTick','on') %'FontSize',5,'units','centimeters','position',[(3.4*y)-2.4 1 3 12])
    %else
        %set(gca,'xtick',[0:2:ceil(max(ric(:,1)))],'XMinorTick','on')
        %end    
        hold off    
        set(gcf,'PaperUnits','centimeters','PaperPositionMode','manual','PaperPosition', [0.1 0.1 ((P-1)/2)*4+2*((P-1)/2) 14]);
        view([90 90])
        end
    end
end

%nota sulla versione: corretta xlabel, aggiungendo tra gli argomenti Ftext
%rispetto a 1_1: cambiate le dimensioni dei grafici, agendo ora su axis
%rispetto a 1_2: risolto il problema con ylabel
%                il grafico ora viene posizionato i basso a sinistra foglio
