%this code is provided as-is without any warranty or indemnification of any kind. 
%for any problem please contact me: e-mail adress: valedido@unina.it
%if you are using it please cite:
%Di Donato,V., Jamka. J., Martn-Fernndez J.A. 2020 - Compositional regression-based methods for SST reconstruction
%Alpine and Mediterranean Quaternary,...

%plots the palaeoenvironmental reconstructions computed with the
%codatransfer function
%How to use:
%ric            is the output of codatransfer function
%rotazione      can be 'oriz' or 'vert': 'oriz' makes separate graphs with
%               the age (depht) as x axis; 'vert' makes joined subplot graphs, with
%               the age (depht) as y axis
%Mtext          is the cell array with the column headers of the M file(see MAT function)
%depthlabel     is the label for the depht axis (i.e. 'cm' or 'age cal ka'))
%RMSE           is the RMSE (root mean square error) output of the codatransfer function
%oggi           is a string with the present values of the environmental
%               parameters. It is optional and can be an empty matrix. In this case
%               the anomalies graph is obtained as a difference from the
%               topmost recontructed values
%
%example:       plotricreg(ricfit,'vert',Mtext,'age cal ka',RMSE,[])
function plotricreg(ric,rotazione,Mtext,depthlabel,RMSE,oggi)
[L,P]=size(ric);
%crea una copia di ric con le anomalie invece dei valori assoluti
%se c' "oggi" lo usa per calcolare le anomalie
if nargin==6 && isempty(oggi)==0
    ric_anomalie=ric;
    for j=2:P
    ric_anomalie(:,j)=ric_anomalie(:,j)-oggi(1,j-1);
    end
%se c' "oggi" ma  vuota ricava dal primo campione 
elseif nargin==6 && isempty(oggi)==1
%prima era:
    %elseif nargin==6 || isempty(oggi)==0
    ric_anomalie=ric;
    for j=2:P
    ric_anomalie(:,j)=ric_anomalie(:,j)-ric(1,j);
    end
elseif nargin==5 
    ric_anomalie=ric;
    for j=2:P
    ric_anomalie(:,j)=ric_anomalie(:,j)-ric(1,j);
    end    
end
%minimo dei minimi
minmin=min(min(ric(:,2:end)));
minmina=min(min(ric_anomalie(:,2:end)));
%massimo dei massimi
maxmax=max(max(ric(:,2:end)));
maxmaxa=max(max(ric_anomalie(:,2:end)));
    %crea i grafici
    switch rotazione
    %grafici singoli
    case 'oriz'
        for j=1:P-1
        figure
        plot(ric(:,1),ric(:,j+1),'Linewidth',1.5);
        %invece di ric(1,1) prima c'era 0
        axis([ric(1,1) ceil(max(ric(:,1))) -inf inf]);
        ylabel(Mtext(1,j+3));
        xlabel(depthlabel);       
        ax=[ric(1,1)+2 ric(1,1)+2];
        ay=[mean(ric(:,j)) mean(ric(:,j))+RMSE(1,j)];
        line(ax,ay)
        text(ax+0.1,ay,'RMSE','Fontsize',4)
        hold on
        set(gca,'XTickMode','auto','XMinorTick','on')
    %else
    %set(gca,'xtick',[0:2:ceil(max(ric(:,1)))],'XMinorTick','on')
    %end    
        hold off
        end
        %grafico anomalie
        for j=1:P-1
        figure;
        plot(ric_anomalie(:,1),ric_anomalie(:,j+1),'Linewidth',1.5)
        axis([ric(1,1) ceil(max(ric_anomalie(:,1))) -inf inf]);
        ylabel(Mtext(1,j+3));
        xlabel(depthlabel);
        ax=[ric(1,1)+2 ric(1,1)+2];
        ay=[mean(ric(:,j)) mean(ric(:,j))+RMSE(1,j)];
        line(ax,ay)
        text(ax+0.1,ay,'RMSE','Fontsize',4);
        hold on
        set(gca,'XTickMode','auto','XMinorTick','on')
        %else
        %set(gca,'xtick',[0:2:ceil(max(ric(:,1)))],'XMinorTick','on')
        %end    
        hold off
        end
    %grafici per subplot
    case 'vert'
    %definisce la successione di colori (fino a 9) rosso scuro rosso arancione blu
    %verde nero e nero a seguire
    rossoarabluver=[0.5 0 0; 1 0 0; 1 0.4 0.2; 0 0 1; 0 1 0; 0 0 0; 0 0 0 ; 0 0 0; 0 0 0];
    figure
    %set(gcf,'PaperUnits','centimeters','PaperPositionMode','manual','PaperPosition', [0.1 0.1 ((P-1)/2)*6 14]);
    set(gcf,'DefaultAxesColorOrder',rossoarabluver(1,:))
    %crea il primo subplot
    %subplot(1,P-1,1)
    subplot('position',[0.1 0.1 0.1 0.8])
    hold on
    plot(ric(:,1),ric(:,2),'Linewidth',0.5);
    axis([ric(1,1) ceil(max(ric(:,1))) -inf inf]);
    ylabel(Mtext(1,2));
    xlabel(depthlabel);
    ax=[ric(1,1)+0.5 ric(1,1)+0.5];
    ay=[min(ric(:,2)) min(ric(:,2))+RMSE(1,1)];
    line(ax,ay)
    text(mean(ax)+0.5,mean(ay),'RMSE','Fontsize',4)
    set(gca,'XTickMode','auto','XMinorTick','on','XAxisLocation','bottom','YAxisLocation','right','YMinorTick','on') %FontSize',5,'units','centimeters','position',[1 1 3 12])
    view([90 90])
    %subplot successivi, l'unica differenza  che non appare l'etichetta
    %dell'asse Y (age o cm)
        for j=2:P-1
        set(gcf,'DefaultAxesColorOrder',rossoarabluver(j,:))
        subplot('position',[0.1+(j-1)*0.15 0.1 0.1 0.8])
        hold on
        plot(ric(:,1),ric(:,j+1),'Linewidth',0.5);
        axis([ric(1,1) ceil(max(ric(:,1))) -inf inf]);
        ylabel(Mtext(1,j+1));
        ax=[ric(1,1)+0.5 ric(1,1)+0.5];
        ay=[min(ric(:,j+1)) min(ric(:,j+1))+RMSE(1,j)];
        line(ax,ay)
        text(mean(ax)+0.5,mean(ay),'RMSE','Fontsize',4)
        xticklabels({})
        set(gca,'XTickMode','auto','XMinorTick','on','XAxisLocation','bottom','YAxisLocation','right',...
            'YMinorTick','on') %'FontSize',5,'units','centimeters','position',[(3.4*y)-2.4 1 3 12])
        hold off    
        %set(gcf,'PaperUnits','centimeters','PaperPositionMode','manual','PaperPosition', [0.1 0.1 ((P-1)/2)*4+2*((P-1)/2) 14]);
        view([90 90])
        set(gcf, 'units','centimeters','PaperPosition', [1 1 25 15]);
        end
        %grafico anomalie
    figure
    %set(gcf,'PaperUnits','centimeters','PaperPositionMode','manual','PaperPosition', [0.1 0.1 ((P-1)/2)*6 14]);
    set(gcf,'DefaultAxesColorOrder',rossoarabluver(1,:))
    subplot('position',[0.1 0.1 0.1 0.8])
%    subplot(1,P-1,1)
    hold on
    plot(ric_anomalie(:,1),ric_anomalie(:,2),'Linewidth',0.5);
    axis([ric(1,1) ceil(max(ric(:,1))) -inf inf]);
    ylabel(Mtext(1,2));
    xlabel(depthlabel);
    ax=[ric(1,1)+0.5 ric(1,1)+0.5];
    ay=[min(ric_anomalie(:,2)) min(ric_anomalie(:,2))+RMSE(1,1)];
    line(ax,ay)
    text(mean(ax)+0.5,mean(ay),'RMSE','Fontsize',4)
    xticklabels({})
    set(gca,'XTickMode','auto','XMinorTick','on','XAxisLocation','bottom','YAxisLocation','right','YMinorTick','on') %FontSize',5,'units','centimeters','position',[1 1 3 12])
    view([90 90])
    %subplot successivi, l'unica differenza  che non appare l'etichetta
    %dell'asse Y (age o cm)
        for j=2:P-1
        set(gcf,'DefaultAxesColorOrder',rossoarabluver(j,:))
%        subplot(1,(P-1)/2,y)
        subplot('position',[0.1+(j-1)*0.15 0.1 0.1 0.8])
        hold on
        plot(ric_anomalie(:,1),ric_anomalie(:,j+1),'Linewidth',0.5);
        axis([ric_anomalie(1,1) ceil(max(ric(:,1))) -inf inf]);
        ylabel(Mtext(1,j+1));
        ax=[ric_anomalie(1,1)+0.5 ric_anomalie(1,1)+0.5];
        ay=[min(ric_anomalie(:,j+1)) min(ric_anomalie(:,j+1))+RMSE(1,j)];
        line(ax,ay)
        text(mean(ax)+0.5,mean(ay),'RMSE','Fontsize',4)
        set(gca,'XTickMode','auto','XMinorTick','on','XAxisLocation','bottom','YAxisLocation','right','YMinorTick','on') %'FontSize',5,'units','centimeters','position',[(3.4*y)-2.4 1 3 12])
    %else
        %set(gca,'xtick',[0:2:ceil(max(ric(:,1)))],'XMinorTick','on')
        %end    
        hold off    
        set(gcf,'PaperUnits','centimeters','PaperPositionMode','manual','PaperPosition', [0.1 0.1 ((P-1)/2)*4+2*((P-1)/2) 14]);
        view([90 90])
        end
    end
end

%nota sulla versione: corretta xlabel, aggiungendo tra gli argomenti Ftext
%rispetto a 1_1: cambiate le dimensioni dei grafici, agendo ora su axis
%rispetto a 1_2: risolto il problema con ylabel
%                il grafico ora viene posizionato i basso a sinistra foglio
