%this code is provided as-is without any warranty or indemnification of any kind. 
%for any problem please contact me: e-mail adress: valedido@unina.it
%if you are using it please cite:
%Di Donato,V., Jamka. J., Martn-Fernndez J.A. 2020 - Compositional regression-based methods for SST reconstruction
%Alpine and Mediterranean Quaternary,... 

%Check of the efficiency of the MAT by comparing the measured environmental parameters 
%with the corresponding values estimated from the best analogs.
%
%Mod               is the modern database (see the mat function code help
%                  for details) 
%param             is the number of environmental parameters included in
%                  Mod (see the mat function code help for details)
%analogs           is the number of analogs
%Mtext             is the text string which refer to Mod (see the mat function code help for details)
%we_or_un          can be 'un' (mean) or 'we' (mean weighted on the inverse of the distance)
%dist              is the distance measure, cen be 'euclidean' or
%                  'mahalanobis'
%tipodati          the kind of data included for the assemblages. It is adopted 
%                  for the computation of the suspicious non analogs. Can be: 'ilr', 
%                  'clr', 'reali' (real numbers), composizionali' (i.e. percentages)

%OUTPUT
%ric               are the reconstructed values
%RMSEP             root mean square error if prediction
%MSD               mean square distances (see the codamat paper, Di Donato et al., 2018, AMQ) 
%latAn    		latitude of modern analogs as they appear in Ds
%longAn   		longitude of modern analogs as they appear in Ds
%tab            a table with correlation between measured and estimated values and the mean std 
%CorrIND        multivariate correlation index
%Dsmod    		Analogs distances ordered in ascending order
%IXmod     		running number of the analogs as they appear in Ds

%example
%[ricun,RMSEPun,MSDun,latAnun,longAnun,tabun,CorrINDun,Dsmod,IXmod]=checkMOD(modv2jef,6,6,Mtext,'un','euclidean','clr')

function [ric,RMSEP,MSD,latAn,longAn,tab,CorrIND,Ds,IX]=checkMOD(Mod,param,analogs,Mtext,we_or_un,dist,tipodati)
M=Mod;
MF=[M(:,1) M(:,(4+param):end)];
[m mm]=size(MF);
switch we_or_un
    case 'we'
    switch dist
        case 'euclidean'    
        [Ds, IX, latAn, longAn, ric]=mat(MF,M,param,'euclidean',analogs,'we',tipodati,0.999,'tutti','not1');
        case 'mahalanobis'
        [Ds, IX, latAn, longAn, ric]=mat(MF,M,param,'mahalanobis',analogs,'we',tipodati,0.999,'tutti','not1'); 
    end
    case 'un'
    switch dist
        case 'euclidean'    
        [Ds,IX,latAn,longAn,ric]=mat(MF,M,param,'euclidean',analogs,'un',tipodati,0.999,'tutti','not1');
        case 'mahalanobis'
        [Ds,IX,latAn,longAn,ric]=mat(MF,M,param,'mahalanobis',analogs,'un',tipodati,0.999,'tutti','not1');
    end
end
%REM plotanalog(Ds(1,:),10);
%REM mapanalog(1,latAn,longAn,10,'low',IX);
%REM hold off;
%REM plotric(ric,Mtext);
%REM hold off;

rossoarabluver=[0 0 0; 1 0 0; 1 0.4 0.2; 0 0 1; 0 1 0; 0.49 1 0.83];
for i=1:6
    colore{i}=rossoarabluver(i,:);
end

for u=2:2:2*param
   figure
   errorbar(M(:,3+u./2),ric(:,u),ric(:,u+1),'*','Color',[colore{u/2}])
   title(Mtext(1,3+u./2))
   xlabel('measured');
   ylabel('estimated');
   %axis([-5 30 -5 30])
   set(gca,'XTickMode','auto','XMinorTick','on','XAxisLocation','bottom','YAxisLocation','right','YTickMode','auto','YMinorTick','on','FontName','arial')
   set(gcf,'PaperPosition', [0 0 8 8]);
end

COR=corrcoef([ric(:,2:2:(param*2)),M(:,4:(4+param-1))]);
tab=[COR(1,1+param)
    mean(ric(:,3))];
for i=2:param
    AA=[COR(i,i+param); mean(ric(:,i*2+1))];
    tab=[tab AA];
end

%calcola il valore MSD
misurati=M(:,4:3+param);
stimati=ric(:,2:2:param*2);
MSD=zeros(1,param+1);
msd=zeros(m,param+1);
for j=1:param
    for i=1:m
    %prima gli univariati
    msd(i,j)=(M(i,3+j)-ric(i,j*2)).^2;
    end
    MSD(1,j)=sum(msd(:,j))./m;
end
%il multivariato c=sum((a-b).^2)
    for i=1:m
    msd(i,param+1)=sum((M(i,4:3+param)-ric(i,2:2:param*2)).^2);
    end
    MSD(1,param+1)=sum(msd(:,param+1))./m;
%indice di correlazione multivariato
    CorrIND=sum(tab(1,:));
    
 %calcola la RMSEP
%calcola le differenze al quadrato tra ricostruite e osservate
for i=1:param
diff(:,i)=M(:,3+i)-ric(:,i*2);
end
sqdiff=diff.^2;
mediasqdiff=mean(sqdiff);
RMSEP=mediasqdiff.^0.5;
end
%nota sulla versione
% 1. lo script  trasformato in una funzione
% 2. non  pi necessario inserire una matrice MF.
% 3.aggiunto titolo ed etichetta degli assi ai grafici
% 4.aggiunto output della tabbella delle correlazioni
%nota su 1.2
%aggiunti MSD e CORRMulti
