%this code is provided as-is without any warranty or indemnification of any kind. 
%for any problem please contact me: e-mail adress: valedido@unina.it
%if you are using it please cite:
%Di Donato,V., Jamka. J., Martn-Fernndez J.A. 2020 - Compositional regression-based methods for SST reconstruction
%Alpine and Mediterranean Quaternary,...

%Check of the efficiency of the MAT by comparing the measured environmental parameters 
%with the corresponding values estimated from the best analogs.
%This version of checkmod repeats the analysis for an increasing number of
%analogues from 1 to 'analogs'

%Mod               is the modern database (see the mat function code help
%                  for details) 
%param             is the number of environmental parameters included in
%                  Mod (see the mat function code help for details)
%analogs           is the max number of analogs
%we_or_un          can be 'un' (mean) or 'we' (mean weighted on the inverse of the distance)
%dist              is the distance measure, cen be 'euclidean' or
%                  'mahalanobis'
%tipodati          the kind of data included for the assemblages. It is adopted 
%                  for the computation of the suspicious non analogs. Can be: 'ilr', 
%                  'clr', 'reali' (real numbers), composizionali' (i.e. percentages)

%OUTPUT
%CORtab        a table with correlation indexes (for
%              increasing number of analogs) 
%meanSTDtab    a table with the mean std of the reconstructed values
%MSDtab        a table with the mean square distances (see the codamat paper, Di Donato et al., 2018, AMQ) 
%CorrINDan     a table with multivariate correlation indexes 
%RMSEP         a table with the root mean square error if prediction

%example      [CORtab,meanSTDtab,MSDtab,CorrINDan, RMSEP]=checkmodan(modv5jef,6,15,'we','euclidean','clr')
function [CORtab,meanSTDtab,MSDtab,CorrINDan,RMSEP]=checkmodan(Mod,param,analogs,we_or_un,dist,tipodati)
%rispetto a checkmodan1_0 sfrutta la funzione MAT mentre 
%checkmodan1_0 fa tutto autonomamente
M=Mod;
MF=[M(:,1) M(:,(4+param):end)];
[m mm]=size(MF);
%comincia con un analogo
switch we_or_un
    case 'we'
    switch dist
        case 'euclidean'
        [Ds, IX, latAn, longAn, ric]=mat(MF,M,param,'euclidean',1,'we',tipodati,0.999,'tutti','not1');
        case 'mahalanobis'
        [Ds, IX, latAn, longAn, ric]=mat(MF,M,param,'mahalanobis',1,'we',tipodati,0.999,'tutti','not1'); 
    end
        case 'un'
    switch dist
        case 'euclidean'    
        [Ds, IX, latAn, longAn, ric]=mat(MF,M,param,'euclidean',1,'un',tipodati,0.999,'tutti','not1');
        case 'mahalanobis'
        [Ds, IX, latAn, longAn, ric]=mat(MF,M,param,'mahalanobis',1,'un',tipodati,0.999,'tutti','not1');
    end
end
%REM plotanalog(Ds(1,:),10);
%REM mapanalog(1,latAn,longAn,10,'low',IX);
%REM hold off;
%REM plotric(ric,Mtext);
%REM hold off;
%grafici delle correlazioni (rimosso)
%REM    for u=2:2:2*param
%REM    figure
%REM    errorbar(M(:,3+u./2),ric(:,u),ric(:,u+1),'*')
%REM    title(Mtext(1,3+u./2))
%REM    xlabel('measured');
%REM    ylabel('estimated');
%REM    set(gca,'XTickMode','auto','XMinorTick','on','XAxisLocation','bottom','YAxisLocation','right','FontName','arial')
%REM    set(gcf, 'PaperPosition', [0 0 8 8]);
%REM    end
COR=corrcoef([ric(:,2:2:2*param),M(:,4:4+param-1)]);
CORt=COR(1,1+param);
for i=2:param
CORt=[CORt COR(i,i+param)];
end
CORtab=CORt;

meanSTDtab=[mean(ric(:,3))];
for i=2:param
meanSTDtab=[meanSTDtab mean(ric(:,2*i+1))];
end 

%calcola il valore MSD
misurati=M(:,4:3+param);
stimati=ric(:,2:2:param*2);
MSD=zeros(1,param+1);
msd=zeros(m,param+1);
for j=1:param
    for i=1:m
%prima gli univariati
    msd(i,j)=(M(i,3+j)-ric(i,j*2)).^2;
    end
    MSD(1,j)=sum(msd(:,j))./m;
end
%il multivariato c=sum((a-b).^2) aggiunto come ultima colonna
    for i=1:m
    msd(i,param+1)=sum((M(i,4:3+param)-ric(i,2:2:param*2)).^2);
    end
    MSD(1,param+1)=sum(msd(:,param+1))./m;
    %copia per le repliche su analoghi
    MSDtab=MSD;
%indice di correlazione multivariato %indice di correlazione multivariato NB da 2:5  messo per farlo solo per
%le 4 sst stagionali, se no andrebbe CORt(1,:);
%vicerversa per farlo per tutti ora c' : nel caso si pu rimettere 2:5
    CorrINDan=sum(CORt(1,:));
    
%calcola la RMSEP
%calcola le differenze al quadrato tra ricostruite e osservate
for i=1:param
diff(:,i)=M(:,3+i)-ric(:,i*2);
end
sqdiff=diff.^2;
mediasqdiff=mean(sqdiff);
RMSEP=mediasqdiff.^0.5;
    
%RIPETE TUTTO CON ANALOGHI CRESCENTI
for analo=2:analogs
    switch we_or_un
        case 'we'
            switch dist
            case 'euclidean'
            [Ds, IX, latAn, longAn, ric]=mat(MF,M,param,'euclidean',analo,'we',tipodati,0.999,'tutti','not1');
            case 'mahalanobis'
            [Ds, IX, latAn, longAn, ric]=mat(MF,M,param,'mahalanobis',analo,'we',tipodati,0.999,'tutti','not1'); 
            end
        case 'un'
            switch dist
            case 'euclidean'    
            [Ds, IX, latAn, longAn, ric]=mat(MF,M,param,'euclidean',analo,'un',tipodati,0.999,'tutti','not1');
            case 'mahalanobis'
            [Ds, IX, latAn, longAn, ric]=mat(MF,M,param,'mahalanobis',analo,'un',tipodati,0.999,'tutti','not1');
            end
        end
%REM plotanalog(Ds(1,:),10);
%REM mapanalog(1,latAn,longAn,10,'low',IX);
%REM hold off;
%REM plotric(ric,Mtext);
%REM hold off;
%REM     for u=2:2:2*param
%REM     figure
%REM     errorbar(M(:,3+u./2),ric(:,u),ric(:,u+1),'*')
%REM     title(Mtext(1,3+u./2))
%REM     xlabel('measured');
%REM     ylabel('estimated');
%REM     set(gca,'XTickMode','auto','XMinorTick','on','XAxisLocation','bottom','YAxisLocation','right','FontName','arial')
%REM     set(gcf, 'PaperPosition', [0 0 8 8]);
%REM     end
    COR=corrcoef([ric(:,2:2:2*param),M(:,4:4+param-1)]);
CORt=COR(1,1+param);
for i=2:param
CORt=[CORt COR(i,i+param)];
end
CORtab=[CORtab
        CORt];
meanSTDtab2=[mean(ric(:,3))];
for i=2:param
meanSTDtab2=[meanSTDtab2 mean(ric(:,2*i+1))];
end 
meanSTDtab=[meanSTDtab
            meanSTDtab2];
%calcola il valore MSD
    misurati=M(:,4:3+param);
    stimati=ric(:,2:2:param*2);
    MSD=zeros(1,param+1);
    msd=zeros(m,param+1);
    for j=1:param
        for i=1:m
%prima gli univariati
        msd(i,j)=(M(i,3+j)-ric(i,j*2)).^2;
        end
    MSD(1,j)=sum(msd(:,j))./m;
    end
%il multivariato c=sum((a-b).^2) aggiunto come ultima colonna
    for i=1:m
    msd(i,param+1)=sum((M(i,4:3+param)-ric(i,2:2:param*2)).^2);
    end
    MSD(1,param+1)=sum(msd(:,param+1))./m;
    MSDtab=[MSDtab
        MSD];
%indice di correlazione multivariato NB da 2:5  messo per farlo solo per
%le 4 sst stagionali, se no andrebbe CORt(1,:))
%ora c' :, nel caso si pu rimettere 2:5
    CorrINDan=[CorrINDan 
        sum(CORt(1,:))];

%calcola la RMSEP
%calcola le differenze al quadrato tra ricostruite e osservate
for i=1:param
diff(:,i)=M(:,3+i)-ric(:,i*2);
end
sqdiff=diff.^2;
mediasqdiff=mean(sqdiff);
RMSEP=[RMSEP
    mediasqdiff.^0.5];
end
    numprogr=(1:1:analogs)';
    %aggiunge la prima colonna a tutto
    CORtab=[numprogr CORtab];
    meanSTDtab=[numprogr meanSTDtab];
    MSDtab=[numprogr MSDtab] ;
    CorrINDan=[numprogr CorrINDan];
    RMSEP=[numprogr RMSEP];
    
    %crea i grafici di riepilogo
    rossoarabluver=[0 0 0; 1 0 0; 1 0.4 0.2; 0 0 1; 0 1 0; 0.49 1 0.83];
set(0,'DefaultAxesColorOrder',rossoarabluver);
%grafico delle correlazioni
figure
    plot(CORtab(:,1),CORtab(:,2:end))
    axis([1 analogs 0.96 1]);
    set(gca,'XTick',1:1:analogs)
    ylabel('correlation index');
    xlabel('analogs');
%grafico dell'indice di correlazione multivariato
figure
    plot(CorrINDan(:,1),CorrINDan(:,2),'k')
    %NB param-2  messo per farlo solo per
%le 4 sst stagionali, se no andrebbe param
    axis([1 analogs param-2-0.2 param-2]);
    set(gca,'XTick',1:1:analogs)
    ylabel('correlation index (multivariate)');
    xlabel('analogs');
%grafico delle std
figure
    plot(meanSTDtab(:,1),meanSTDtab(:,2:end))
    axis([1 analogs 0 inf]);
    set(gca,'XTick',1:1:analogs)
    ylabel('standard deviations');
    xlabel('analogs');
%grafico degli indici MSD univariati
figure
    plot(MSDtab(:,1),MSDtab(:,2:2+param-1))
    axis([1 analogs 0 inf]);
    set(gca,'XTick',1:1:analogs)
    ylabel('MSD index');
    xlabel('analogs');
%grafico dell'indice MSD multivariato
figure
    plot(MSDtab(:,1),MSDtab(:,end),'k')
    axis([1 analogs 4 inf]);
    set(gca,'XTick',1:1:analogs)
    ylabel('MSD index (multivariate)');
    xlabel('analogs');
 %grafico dei RMSEP
 figure
    plot(RMSEP(:,1),RMSEP(:,2:end))
    axis([1 analogs 0 inf]);
    set(gca,'XTick',1:1:analogs)
    ylabel('roor mean squared error');
    xlabel('analogs');
end
    

%nota sulla versione
% 1. lo script  trasformato in una funzione
% 2. non  pi necessario inserire una matrice MF.
% 3.aggiunto titolo ed etichetta degli assi ai grafici
% 4.aggiunto output della tabbella delle correlazioni
%nota su 1.2
%aggiunti MSD e CORRMulti
